# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil
from textwrap import dedent

import fixtures

from dbus.exceptions import DBusException
from subprocess import check_call

from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.addressbook import dbus_service

CONTACTS_DB_DIR = '~/.local/share/evolution/addressbook/system/'


class AddressBookTestEnvironment(fixtures.Fixture):
    """Fixture used to configure and clean up the address book environment."""

    def setUp(self):
        super().setUp()
        self.useFixture(BackupContacts())
        self.useFixture(UseEmptyContacts())


class BackupContacts(fixtures.Fixture):

    """Backup contacts when the fixture is used. Restore them in cleanup."""

    def __init__(self):
        """Initialize the fixture."""
        super().__init__()

    def setUp(self):
        super().setUp()
        self.tempdir_fixture = self.useFixture(fixtures.TempDir())
        contacts_database_path = (
            os.path.join(os.path.expanduser(CONTACTS_DB_DIR), 'contacts.db'))
        shutil.copy(contacts_database_path, self.tempdir_fixture.path)
        self.addCleanup(self._restore_database)

        self._restart_service()

    def _restore_database(self):
        shutil.copy(
            os.path.join(self.tempdir_fixture.path, 'contacts.db'),
            os.path.expanduser(CONTACTS_DB_DIR))

        self._restart_service()

    def _restart_service(self):
        check_call(['initctl', 'restart', 'address-book-service'])
        wait_until(self._is_address_book_service_running)

    def _is_address_book_service_running(self):
        """Return True when dbus service is running, False otherwise."""
        try:
            dbus_service.ContactsDbusService()
            return True
        except DBusException:
            return False


class UseEmptyContacts(fixtures.Fixture):
    """Start with an empty contact database."""

    def __init__(self):
        super().__init__()
        self.addressbook_dbus = dbus_service.ContactsDbusService()

    def setUp(self):
        super().setUp()
        self.addressbook_dbus.delete_all_contacts()


class AddTemporaryContacts(fixtures.Fixture):

    def __init__(self, contacts):
        super().__init__()
        self.contacts = contacts
        self.addressbook_dbus = dbus_service.ContactsDbusService()

    def setUp(self):
        super().setUp()
        for contact in self.contacts:
            self.create_temporary_contact(contact)

    def create_temporary_contact(self, contact):
        """
        Create a contact in the address book using the dbus service
        and remove it in the cleanup step.
        """
        vcard = dedent(
            """\
            BEGIN:VCARD
            VERSION:3.0
            FN:{name}
            TEL;TYPE=CELL:{number}
            END:VCARD""".format(name=contact.name, number=contact.number))
        contact_vcard = self.addressbook_dbus.create_contact(vcard)
        contact_uid = self.addressbook_dbus.get_vcard_uid(contact_vcard)
        self.addCleanup(self.addressbook_dbus.delete_contact, [contact_uid])
