# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys
from contextlib import contextmanager

import ubuntuuitoolkit
from autopilot import introspection
from autopilot.exceptions import StateNotFoundError

from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers import wait_until

CO_ORD_MAX = (sys.maxsize, sys.maxsize)


@contextmanager
def override_proxy_timeout(proxy, timeout_seconds):
    original_timeout = proxy._poll_time
    try:
        proxy._poll_time = timeout_seconds
        yield proxy
    finally:
        proxy._poll_time = original_timeout


def _get_pid(pname):
    """ Wait until the process pname is running and retrieve its pid. Raise
    Runtime exception when the process is not started after 10 seconds
    """
    if wait_until(processes.is_process_running, pname):
        return processes.get_process_id(pname)
    else:
        raise RuntimeError('Could not find process: {} '.format(pname))


def validate_dbus_object(path, state, root_name, class_name, **kwargs):
    """Return True if dbus object matches required parameters.
    :param path: Path of object.
    :param state: Properties of object.
    :param root_name: The required root name for the object.
    :param class_name: The required class name for the object.
    :param kwargs: Any other properties to match.
    :return: True if object matches required path root, name and properties,
             False otherwise.
    """
    if (get_path_root(path) == root_name and
            introspection.get_classname_from_path(path) == class_name):
        for key in kwargs.keys():
            if state[key][1] != kwargs[key]:
                return False
        return True
    return False


def get_path_root(object_path):
    """Return the name of the root node of specified path."""
    is_string = isinstance(object_path, str)
    if is_string:
        object_path = object_path.encode('utf-8')
    root = object_path.split(b'/')[1]
    if is_string:
        root = root.decode('utf-8')
    return root


def is_element(predicate, *args, **kwargs):
    """
    Evaluate the predicate with the args and indicate if it raises
    StateNotFoundError
     :param: predicate: The method to be evaluated
     :param: *args: The predicate positional parameters
     :param: **kwargs: The predicate optional parameters
     :return: False if the predicate raises StateNotFoundError, True otherwise
    """
    try:
        predicate(*args, **kwargs)
        return True
    except StateNotFoundError:
        return False


def wait_select_many(root, number=1, timeout=10, period=0.5, type_name='*',
                     **kwargs):
    """
    Get a list of proxy objects matching some search criteria from the
    introspection tree, with type equal to *type_name* and (optionally)
    matching the keyword filters present in *kwargs*. This method will retry
    until either the objects matching the criteria are >= number or the timeout
    is reached
    :param: root: The root object to make the query
    :param: number: The number of objects that have to match to return
    :param: timeout: The timeout for the polling
    :param: period: The retry period of time
    :param: type_name: Either a string naming the type you want, or a class
    of the appropriate type
    :param: **kwargs: The optional parameters used to match objects
    :raise: StateNotFoundError: When the number of objects matching the query
    are less than the number passed as parameter
    :return: A list of proxy objects
    """

    if wait_until(
            lambda: len(root.select_many(type_name, **kwargs)) >= number,
            timeout=timeout, period=period):
        return root.select_many(type_name, **kwargs)
    else:
        raise StateNotFoundError("Not found the number of elements requested")


def get_patched_registry(class_object, class_name):
    """
    Return a patched copy of autopilot's object registry.

    Using the specified class name and class object reference patch autopilot's
    object registry. This can be used in conjunction with autopilot's
    patch_registry() function to ensure that the correct CPO is returned for
    a specified type.

    :param class_object: Object type to create.
    :param class_name: The name of the class to patch.

    """
    registry = introspection._object_registry._object_registry.copy()
    for base_uuid in registry.keys():
        if class_name in registry[base_uuid]:
            registry[base_uuid][class_name] = class_object
    return registry


def _patched_select(wait, root, class_object, class_name=None, **kwargs):
    """
    Patch the autopilot object registry and select an object from root.

    This method is used to avoid autopilot CPO errors by patching the autopilot
    object registry before calling the required select method on the specified
    root object.

    :param wait: Whether to call wait_select_single or select_single.
    :param root: Root introspection object.
    :param class_object: Instance of object to patch the registry with.
    :param class_name: Name of registry entry to patch. If None then the class
                       name of class_object will be used.
    :return: Intropsection object of type class_object.

    """
    if not class_name:
        class_name = class_object.__class__.__name__
    registry = get_patched_registry(class_object, class_name)
    with introspection._object_registry.patch_registry(registry):
        if wait:
            return root.wait_select_single(class_name, **kwargs)
        else:
            return root.select_single(class_name, **kwargs)


def patched_select_single(root, class_object, class_name=None, **kwargs):
    """
    Perform select_single on root object using patched autopilot registry.

    :param root: Root introspection object.
    :param class_object: Instance of object to patch the registry with.
    :param class_name: Name of registry entry to patch. If None then the class
                       name of class_object will be used.
    :return: Intropsection object of type class_object.

    """
    return _patched_select(False, root, class_object, class_name, **kwargs)


def patched_wait_select_single(root, class_object, class_name=None, **kwargs):
    """
    Perform wait_select_single on root object using patched autopilot registry.

    :param root: Root introspection object.
    :param class_object: Instance of object to patch the registry with.
    :param class_name: Name of registry entry to patch. If None then the class
                       name of class_object will be used.
    :return: Intropsection object of type class_object.

    """
    return _patched_select(True, root, class_object, class_name, **kwargs)


def get_proxy_object(pid):
    return introspection.get_proxy_object_for_existing_process(
        pid=pid,
        emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase,
    )


def get_proxy_object_for_existing_process(
        process_name, base_custom_proxy_object):
    """Return the proxy object for a currently running process."""
    return introspection.get_proxy_object_for_existing_process(
        pid=_get_pid(process_name),
        emulator_base=base_custom_proxy_object
    )


def get_proxy_object_for_existing_app(
        process_name, cleanup_process=None,
        base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """
    Return proxy object from existing application process.

    Add cleanup action to close the application using cleanup_process
    parameter. If cleanup_process is None then process_name is used as the
    cleanup target using ensure_application_closed().

    :param pname: Name of required application process.
    :param cleanup_process: Name of process to clean up.
    :param base: custom emulator base for proxy object
    :return: Proxy object for the desired application.

    """
    if not cleanup_process:
        cleanup_process = process_name
    proxy = get_proxy_object_for_existing_process(process_name, base)
    context.shared.add_cleanup(processes.ensure_application_closed,
                               cleanup_process)
    return proxy


def get_job_proxy_object(pname):
    pid = _get_pid(pname)
    return get_proxy_object(pid)


def get_proxy_object_for_existing_qmlscene_process(qmlfile):
    """
    Return proxy object from existing qmlscene process.

    Add cleanup action to close the application using cleanup_process
    parameter. If cleanup_process is None then process_id is used as the
    cleanup target using processes.stop_qmlscene_process().

    :param qmlfile: Thje name of the QML file for the required process
    :return: Proxy object for the desired qmlscene process

    """
    process_id = processes.get_qmlscene_process_id(qmlfile)
    proxy = introspection.get_proxy_object_for_existing_process(
        pid=process_id,
        emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase
    )
    context.shared.add_cleanup(processes.stop_qmlscene_process, qmlfile)
    return proxy


def click_object(func):
    """Wrapper which clicks the returned object"""
    def func_wrapper(self, *args, **kwargs):
        return self.pointing_device.click_object(func(self, *args, **kwargs))

    return func_wrapper
