# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should  received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import time

from autopilot.exceptions import StateNotFoundError

from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.autopilot import CO_ORD_MAX


def _get_y_and_x(item):
    """
    Return y and x co-ordinates for specified object.

    :param item: Item to check
    :return: (y, x) co-ordinates
    """
    try:
        # If item is no longer valid this can cause excpetion
        co_ords = item.globalRect.y, item.globalRect.x
    except StateNotFoundError:
        # Trying to sort an object that no longer exists
        # return a dummy key value so this item is sorted last
        co_ords = CO_ORD_MAX
    return co_ords


def _get_x_and_y(item):
    """
    Return x and y co-ordinates for specified object.

    :param item: Item to check
    :return: (x, y) co-ordinates
    """
    try:
        # If item is no longer valid this can cause excpetion
        co_ords = item.globalRect.x, item.globalRect.y
    except StateNotFoundError:
        # Trying to sort an object that no longer exists
        # return a dummy key value so this item is sorted last
        co_ords = CO_ORD_MAX
    return co_ords


def get_object_path(item):
    """Return the object path of a specified item."""
    return item._path.decode('utf-8')


def wait_until_is_not_moving(elem, move_time=0.5, timeout=10):
    """ Wait until the autopilot element is not moving based on its globalRect
    coordinates
    :param elem: autopilot element to check
    :param move_time: time in seconds between the coordinates are retrieved
    :param timeout: total time to wait until the element is not moving
    :return: True when the element is not moving before the timeout is reached,
    otherwise return False
    """
    return wait_until(is_not_moving, elem, move_time, timeout=timeout,
                      period=0)


def is_not_moving(elem, move_time=0.5):
    """ Indicate if the element has not changed its position after a period
    based on its globalRect coordinates
    :param elem: autopilot element to check
    :param move_time: time between the coordinates are retrieved
    :return: True when the x and y coordinates are equal, False otherwise
    """
    x1, y1, h1, w1 = elem.globalRect
    time.sleep(move_time)
    x2, y2, h2, w2 = elem.globalRect

    return x1 == x2 and y1 == y2


def _get_parent_by_type(item, required_type, **kwargs):
    """Return a parent item of the required type.
    :param item: The child item to start searching from.
    :param required_type: String name of parent object type required.
    :param kwargs: This is used for logging purposes only.
    :return: Parent object matching required type.
    :raises: StateNotFoundError if no matching object type can be found.
    """
    parent = item.get_parent()
    path = get_object_path(parent).split('/')
    try:
        index = path.index(required_type)
    except ValueError:
        raise StateNotFoundError(
            '{}, and properties: {}'.format(required_type, kwargs))
    for count in range(1, len(path) - index):
        parent = parent.get_parent()
    return parent


def validate_properties(item, **kwargs):
    """Return True if item properties match specified keyword properties."""
    props = item.get_properties()
    for key in kwargs.keys():
        if props[key] != kwargs[key]:
            return False
    return True


def get_parent(item, required_type, **kwargs):
    """Return a parent object of specified type and properties.
    :param item: The child item to start searching from.
    :param required_type: String name of parent object type required.
    :param kwargs: Any properties to match on the parent object.
    :return: Parent object matching required type and properties.
    :raises: StateNotFoundError if no matching object can be found.
    """
    parent = item
    while True:
        parent = _get_parent_by_type(parent, required_type, **kwargs)
        if validate_properties(parent, **kwargs):
            return parent
        parent = parent.get_parent()
