# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from functools import reduce
from math import sqrt
from operator import add
import os
from PIL import (Image, ImageChops)
import subprocess


def get_colour_count(image_path):
    """Return the number of colours in the specified image."""
    img = Image.open(image_path)
    # get a list of colours from total number of pixels in the image
    colours = img.getcolors(img.size[0] * img.size[1])
    return len(colours)


def _get_exiftool_value(image_path, *values):
    """ Return the exittool value of the picture """
    result = subprocess.check_output(['exiftool', image_path] + list(values) +
                                     ['-s', '-s', '-s'])
    return result.strip()


def get_flash_status_from_exif_data(image_path):
    """Return the bool flash status from the exif data of specified file."""
    result = _get_exiftool_value(image_path, '-Flash', '-n')
    if len(result) > 0:
        status = bool(int(result))
    else:
        status = None
    return status


def get_gps_location_from_exif_data(image_path):
    """Return the gps (lat, long) from the exif data of specified file."""
    return (_get_exiftool_value(image_path, '-gpslatitude', '-n'),
            _get_exiftool_value(image_path, '-gpslongitude', '-n'))


def get_image_from_rgba(path, size):
    """ Retrieve a PIL Image object from a RGBA in a filepath """
    with open(path, "rb") as f:
        image_data = Image.frombuffer("RGBA", size, f.read(), "raw", "RGBA",
                                      0, 1)
    return image_data


def convert_rgba_image(rgba_source_file, target_file, size):
    """
    Convert rgba file to PNG target file
    :param rgba_source_file: Path to rgba source image
    :param target_file: Path to PNG target file
    :param size: Size used for the target image
    """
    try:
        cmd = [
            'convert',
            '-alpha', 'off',
            '-depth', '8',
            '-size', size,
            'rgba:{}[0]'.format(rgba_source_file),
            target_file
        ]
        subprocess.check_call(cmd)
    except FileNotFoundError as e:
        e.args += ("'convert' is not available, install imagemagick.", )
        raise
    except subprocess.CalledProcessError as e:
        e.args += ("Failed to convert screenshot to png format.", )
        raise


def _rms(im):
    """ Calculate the root-mean-square difference of an image """
    h = im.histogram()
    return sqrt(reduce(add, map(lambda h, i: h*(i**2), h, range(256))) /
                (float(im.size[0]) * im.size[1]))


def _rmsdiff(im1, im2):
    """ Calculate the root-mean-square difference between two images """
    return _rms(ImageChops.difference(im1, im2))


def images_almost_equal(im1, im2):
    """ Determine if two images are almost equal or not
    :return: True if the images have almost same content, False otherwise
    """
    return _rmsdiff(im1, im2) < 2


def images_equal(im1, im2):
    """ Determine if two images are equal or not
    :return: True if the images have same content, False otherwise
    """
    return ImageChops.difference(im1, im2).getbbox() is None


def image_files_equal(im1_path, im2_path):
    """ Determine if two image files are equal or not.
    :param im1_path: File path to first image file.
    :param im2_path: File path to second image file.
    :return: True if the image files have same content, False otherwise.
    """
    return images_equal(Image.open(im1_path), Image.open(im2_path))


def get_image_number_of_pixels(im):
    """ Get the number of pixels for an image """
    (width, height) = im.size
    return width * height


def get_image_size(im):
    """ Get the image size (width, height) """
    return im.size


def get_image(path):
    """ Retrieve a PIL.Image object from an image path """
    if not os.path.isfile(path):
        raise RuntimeError('Image path does not exist')
    return Image.open(path)


def create_new_image(src, dst):
    """ Create a new image based on the source image """
    img = Image.open(src)
    img.save(dst)
