
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from unity8 import indicators

from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers.ubuntuuitoolkit._qquicklistview import QQuickListView  # NOQA

INDICATOR_PAGE_PATH = b'IndicatorPage'

BATTERY_INDICATOR = 'indicator-power'
BATTERY_INDICATOR_PAGE = 'indicator-power-page'
BLUETOOTH_INDICATOR = 'indicator-bluetooth'
BLUETOOTH_INDICATOR_PAGE = 'indicator-bluetooth-page'
DATETIME_INDICATOR = 'indicator-datetime'
DATETIME_INDICATOR_PAGE = 'indicator-datetime-page'
NETWORK_INDICATOR = 'indicator-network'
NETWORK_INDICATOR_PAGE = 'indicator-network-page'
SOUND_INDICATOR = 'indicator-sound'
SOUND_INDICATOR_PAGE = 'indicator-sound-page'
MESSAGES_INDICATOR = 'indicator-messages'
MESSAGES_INDICATOR_PAGE = 'indicator-messages-page'
TRANSFER_INDICATOR = 'indicator-transfer'

logger = logging.getLogger(__name__)


class IndicatorPage(indicators.IndicatorPage):
    """Extend IndicatorPage from Unity8."""

    def _get_listview(self):
        """Return the list view object from the indicator."""
        return self.select_single(QQuickListView, objectName='mainMenu')

    def press_label(self, text):
        """Press the label in the list view of the indicator."""
        self._get_listview().press_label(text)


class Indicator:

    def __init__(self, main_window, name, page):
        super().__init__()
        self._main_window = main_window
        self.name = name
        self.page = page

    def open(self):
        """Open the requested indicator page.

        :return: The custom proxy object for the requested indicator page.

        """
        return self._main_window.open_indicator_page(self.name)

    def close(self):
        """Close the indicator page."""
        self._main_window.close_indicator_page()

    def ensure_closed(self):
        """Ensure the indicator page is closed."""
        indicator_page = self._main_window.select_single(objectName=self.page)
        if indicator_page.visible:
            self.close()


def open_indicator(indicator_name, indicator_page_name, return_base=False):
    """
    Pull down the requested indicator and return its CPO
    :param indicator_name: name of the indicator to open
    :param indicator_page_name: object name of the indicator page
    :param return_base: bool representing if the top-level indicator
                        object should be returned
    """
    indicator = Indicator(context.shared.unity.main_window, indicator_name,
                          indicator_page_name)

    indicator_page = indicator.open()
    context.shared.add_cleanup(indicator.ensure_closed)

    if not return_base:
        return indicator_page

    return indicator, indicator_page


def open_battery_indicator(return_base=False):
    """
    Open battery indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along battery indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators import battery  # NOQA
    return open_indicator(BATTERY_INDICATOR, BATTERY_INDICATOR_PAGE,
                          return_base=return_base)


def open_bluetooth_indicator(return_base=False):
    """
    Open bluetooth indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along bluetooth indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators import bluetooth  # NOQA
    return open_indicator(BLUETOOTH_INDICATOR, BLUETOOTH_INDICATOR_PAGE,
                          return_base=return_base)


def open_datetime_indicator(return_base=False):
    """
    Open Datetime indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along datetime indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators import datetime  # NOQA
    return open_indicator(DATETIME_INDICATOR, DATETIME_INDICATOR_PAGE,
                          return_base=return_base)


def open_network_indicator(return_base=False):
    """
    Open network indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along network indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators.network import _cpo  # NOQA
    return open_indicator(NETWORK_INDICATOR, NETWORK_INDICATOR_PAGE,
                          return_base=return_base)


def open_sound_indicator(return_base=False):
    """
    Open sound indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along sound indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators import sound  # NOQA
    return open_indicator(SOUND_INDICATOR, SOUND_INDICATOR_PAGE,
                          return_base=return_base)


def open_message_indicator(return_base=False):
    """
    Open message indicator and return its proxy object
    :param return_base: bool representing if the base indicator should be
                        returned along message indicator proxy
    """
    from ubuntu_system_tests.helpers.indicators import message  # NOQA
    return open_indicator(MESSAGES_INDICATOR, MESSAGES_INDICATOR_PAGE,
                          return_base=return_base)
