# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging

import ubuntuuitoolkit as uuitk

from ubuntu_system_tests.helpers import ofono, unity8
from ubuntu_system_tests.helpers.indicators import(
    IndicatorPage,
    INDICATOR_PAGE_PATH
)
from ubuntu_system_tests.helpers.autopilot import(
    validate_dbus_object,
    wait_select_many,
)


logger = logging.getLogger(__name__)


class NotificationsIndicatorItem(uuitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent 'Notifications' indicator shown in top panel."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, unity8.UNITY8_PATH_ROOT, b'IndicatorItem',
            objectName='indicator-messages-panelItem')

    def is_new_message_displayed(self):
        return 'indicator-messages-new' in self.icons[0]


class NotificationsIndicatorPage(IndicatorPage):
    """Autopilot helper for the messages indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, unity8.UNITY8_PATH_ROOT, INDICATOR_PAGE_PATH,
            objectName='indicator-messages-page')

    def get_message(self, sender, sim='SIM 1', **kwargs):
        """Return the message with specified sender and arguments."""
        title = sender
        if ofono.number_of_sims() > 1:
            title += ' - [{sim}]'.format(sim=sim)

        return self.select_single(
            MessageHeader, title=title, visible=True, **kwargs)

    def get_missed_call(self, caller, **kwargs):
        """Return the missed call with the specified caller."""
        return self.select_single(
            MessageHeader, title=caller, visible=True, **kwargs
        )

    def clear_all(self):
        """Press the 'Clear All' label to clear all existing notifications."""
        clear_all = self._get_listview().swipe_to_find(
            self._get_clear_all_button
        )
        self.pointing_device.click_object(clear_all)

    def _get_clear_all_button(self):
        return self.select_single(
            objectName='indicator.remove-all',
            visible=True
        )

    def _get_reply_input_field(self):
        """Return the sms reply entry."""
        return self.select_single('TextArea', objectName='replyText')

    def _type_message(self, message):
        """Focus the reply entry and type the reply message."""
        reply_field = self._get_reply_input_field()
        reply_field.write(message)

    def _get_send_button(self):
        """Return the reply button."""
        return self.select_single('Button', objectName='sendButton')

    def reply_sms(self, message_body, message_item):
        """
        Expand the message item in the indicator, type reply message and
        send.
        :param message_body: the message text to send
        :param message_item: the message object for the message to reply to.
        """
        self.pointing_device.click_object(message_item)
        self._type_message(message_body)
        self.pointing_device.click_object(self._get_send_button())
        message_item.wait_until_destroyed()

    def get_last_message(self, number):
        """ Get the last message displayed in the indicator """
        return wait_select_many(self, number, objectName='title')[-1].text

    def wait_while_indicator_visible(self):
        """ Wait until the indicator message is not displayed any more """
        messages = self.select_many(objectName='title')
        for message in messages:
            if not message.truncated:
                message.truncated.wait_for(True)


class MessageHeader(uuitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for message object displayed in indicator."""

    def get_call_icon(self):
        """Return the call icon object."""
        return self.select_single(
            'Icon', objectName='icon',
            source='image://theme/telephony-service-call')

    def get_contact_avatar(self):
        """Return the contact's avatar icon."""
        return self.select_single('Icon', objectName='avatar', visible=True)

    @autopilot_logging.log_action(logger.info)
    def call(self):
        """Press the call icon."""
        self.pointing_device.click_object(self.get_call_icon())

    @autopilot_logging.log_action(logger.info)
    def press_avatar(self):
        """Press the contact's avatar to display snap decision menu."""
        self.pointing_device.click_object(self.get_contact_avatar())
