# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import ubuntuuitoolkit as uuitk

from ubuntu_system_tests.helpers.indicators import(
    IndicatorPage,
    INDICATOR_PAGE_PATH
)
from ubuntu_system_tests.helpers.autopilot import(
    validate_dbus_object,
    click_object
)
from ubuntu_system_tests.helpers import unity8
from ubuntu_system_tests.helpers.ubuntuuitoolkit.switchmenu import SwitchMenu
from ubuntu_system_tests.helpers.ubuntuuitoolkit._qquicklistview import QQuickListView  # NOQA


class NetworkIndicatorPage(IndicatorPage):
    """Autopilot helper for the network indicator page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, unity8.UNITY8_PATH_ROOT, INDICATOR_PAGE_PATH,
            objectName='indicator-network-page')

    def is_flight_mode_enabled(self):
        """Return True if flight mode is on, False otherwise."""
        return self._get_flight_mode_switch().checked

    def is_flight_mode_switch_visible(self):
        """Return True if flight mode switch is visible, False otherwise."""
        return self._get_flight_mode_switch().visible

    def _get_flight_mode_switch(self):
        return self.select_single(
            SwitchMenu,
            objectName='indicator.airplane.enabled')

    def turn_on_flight_mode(self):
        """Actuate the flight mode switch to turn on as necessary."""
        if not self.is_flight_mode_enabled():
            self._get_flight_mode_switch().toggle()

    def turn_off_flight_mode(self):
        """Actuate the flight mode switch to turn off as necessary."""
        if self.is_flight_mode_enabled():
            self._get_flight_mode_switch().toggle()

    def is_wifi_enabled(self):
        """Return True if WiFi is enabled, False otherwise."""
        return self._get_wifi_switch().checked

    def is_wifi_switch_visible(self):
        """Return True if WiFi switch is visible, False otherwise."""
        return self._get_wifi_switch().visible

    def _get_wifi_switch(self):
        """Return the WiFi switch."""
        return self.select_single(
            SwitchMenu,
            objectName='indicator.wifi.enable')

    def turn_on_wifi(self):
        """Actuate the WiFi switch to turn on as necessary."""
        if not self.is_wifi_enabled():
            self._get_wifi_switch().toggle()

    def turn_off_wifi(self):
        """Actuate the WiFi switch to turn off as necessary."""
        if self.is_wifi_enabled():
            self._get_wifi_switch().toggle()

    def _get_cellular_settings_menu_item(self):
        """Return the menu item which opens cellular settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.cellular.settings')

    def is_cellular_settings_visible(self):
        return self._get_cellular_settings_menu_item().visible

    @click_object
    def open_cellular_settings(self):
        """Tap the cellular settings list item to open settings."""
        return self._get_cellular_settings_menu_item()

    def _get_wifi_settings_menu_item(self):
        """Return the menu item which opens WiFi settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.wifi.settings')

    def is_wifi_settings_visible(self):
        return self._get_wifi_settings_menu_item().visible

    @click_object
    def open_wifi_settings(self):
        """Tap the wifi settings list item to open settings."""
        list_view = self.select_single(QQuickListView, objectName='mainMenu')
        return list_view.swipe_to_find(self._get_wifi_settings_menu_item)

    def get_list_of_sims(self):
        """Return a list of ModemInfoItems, or empty list if none found.

        Note that we don't count the 'No SIM' item which appears if no
        SIM is found.

        """
        return sorted(
            [sim for sim in self.select_many('ModemInfoItem')
             if sim.is_inserted()],
            key=lambda x: x.globalRect.y
        )

    def is_sim_list_offline(self):
        """Return True if all SIMs are offline, False otherwise."""
        return all([sim.is_offline() for sim in self.get_list_of_sims()])

    def get_list_of_wifi_access_points(self):
        """Return a list of WiFi access points, empty list if none found."""
        return self.select_many('AccessPointMenu')

    def get_number_of_wifi_access_points(self):
        """Return total number of visible wifi access points."""
        return len(self.get_list_of_wifi_access_points())


class ModemInfoItem(uuitk.UbuntuUIToolkitCustomProxyObjectBase):

    def is_offline(self):
        return self.statusText == 'Offline'

    def is_inserted(self):
        """Return False if SIM ModemInfoItem reads 'No SIM', True otherwise.

        The SIM list is never empty--a dummy ModemInfoItem with text
        'No SIM' appears if no SIMs are inserted.  This function may
        be used for counting of *actual* SIMs, e.g.

        """
        return self.statusText != 'No SIM'

    def get_sim_name(self):
        """Returns the name of the SIM, is mostly the provider name."""
        return self.statusText
