# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase

from ubuntu_system_tests.helpers.autopilot import validate_dbus_object

APP_PATH_ROOT = b'MediaPlayer'


class MediaPlayer(UbuntuUIToolkitCustomProxyObjectBase):
    """Root CPO for Media Player process."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, APP_PATH_ROOT,
            applicationName='Media Player')

    @property
    def player(self):
        """Return VideoPlayer object."""
        return self.select_single(objectName='player')


class VideoPlayer(UbuntuUIToolkitCustomProxyObjectBase):
    """CPO for Media Player video player window."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'VideoPlayer',
            objectName='player')

    @property
    def remaining_time(self):
        """Return remaining playback time of current video."""
        return self.get_timeline().remainingTime

    def get_toolbar(self):
        """Reutn ToolBar object."""
        return self.select_single(objectName='toolbar')

    def get_controls(self):
        """Return Controls object."""
        return self.get_toolbar().select_single(objectName='controls')

    def get_timeline(self):
        """Return TimeLine object."""
        return self.get_toolbar().select_single(objectName='TimeLine.Slider')

    def get_playback_button(self):
        """Return play/pause button."""
        return self.get_controls().select_single(
            objectName='Controls.PlayBackButton')

    def show_controls(self):
        """Display the playback controls if they are currently hidden."""
        if not self.controlsActive:
            self.pointing_device.click_object(self)
            self.get_toolbar().active.wait_for(True)

    def hide_controls(self):
        """Hide the playback controls if they are currently visible."""
        if self.controlsActive:
            self.pointing_device.click_object(self)
            self.get_toolbar().active.wait_for(False)

    def is_playing(self):
        """Return True if video has not been fully played, False otherwise."""
        return self.remaining_time != '00:00:00'

    def is_play_finished(self):
        """Return True if playback is completed, False otherwise."""
        return self.remaining_time == '00:00:00'

    def play(self):
        """Play video if it is not currently playing."""
        if not self.playing:
            self.show_controls()
            self.pointing_device.click_object(self.get_playback_button())
            self.playing.wait_for(True)

    def pause(self):
        """Pause video if it is not currently paused."""
        if not self.paused:
            self.show_controls()
            self.pointing_device.click_object(self.get_playback_button())
            self.paused.wait_for(True)
