# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import logging

from threading import Timer

from ubuntu_system_tests.helpers.dbus_observer import DBusObserver
from ubuntu_system_tests.helpers.threads import StoppingThread

logger = logging.getLogger(__name__)


class NotificationObserver(DBusObserver):
    """Class to observe when new messages are received."""

    USSD_INTERFACE = 'com.canonical.Telephony.USSD'
    USSD_RECEIVED_SIGNAL = 'NotificationReceived'

    NOTIFICATION_INTERFACE = 'org.freedesktop.Notifications'
    NOTIFICATION_CLOSED_SIGNAL = 'NotificationClosed'

    def __init__(self, stop, result, notifications, expected_count, timeout):
        """
        Construct the NotificationObserver object.

        :param stop: Event object to signal when to stop waiting.
        :param result: Event object to signal when all expected notifications
                       have been observed.
        :param notifications: Notifications object which is used to introspect
                              the notifications being displayed.
        :param expected_count: The expected number of notification objects
                               that are expected to be displayed.
        :param timeout: How long to wait for the notifications to appear.

        """
        super().__init__(stop, result)
        self.bus = dbus.SessionBus()
        self.notifications = notifications
        self.remaining = expected_count
        self.timeout = timeout
        self._add_signal_receivers()

    def _add_signal_receivers(self):
        """Add required dbus signal receivers."""
        self.bus.add_signal_receiver(
            self._on_notification_closed,
            dbus_interface=self.NOTIFICATION_INTERFACE,
            signal_name=self.NOTIFICATION_CLOSED_SIGNAL)

        self.bus.add_signal_receiver(
            self._on_ussd_message_received,
            dbus_interface=self.USSD_INTERFACE,
            signal_name=self.USSD_RECEIVED_SIGNAL)

    def start(self):
        """Start the notification observer."""
        self.timer = Timer(self.timeout, self._on_timeout)
        self.timer.start()
        super().start()

    def stop(self):
        """Stop the notification observer."""
        self.timer.cancel()
        self.result.set()
        super().stop()

    def _on_timeout(self):
        """Callback for when timeout occurs."""
        logger.info('Notification observer timeout reached.')
        self.stop()

    def _cleanup(self):
        """Execute cleanup actions before the observer stops."""
        self.bus.remove_signal_receiver(self._on_notification_closed)
        self.bus.remove_signal_receiver(self._on_ussd_message_received)

    def _on_notification_closed(self, *args, **kwargs):
        """Callback for when a displayed notification is closed."""
        self.remaining -= 1
        if self.remaining == 0 and not self.result.is_set():
            logger.info('All expected notifications closed')
            self.result.set()

    def _on_ussd_message_received(self, *args, **kwargs):
        """Callback for when a new ussd message is received."""
        # _on_notification_closed is called twice for each ussd message closed
        self.remaining += 2
        self._dismiss_ussd_notification()

    def _dismiss_ussd_notification(self):
        """Dismiss the USSD notification being displayed."""
        self.notifications.get_provider_notification(
            timeout=30, stop_event=self._stop).press_ok()


def _start_observer(stop, result, notifications, expected_count,
                    timeout):
    """
    Start a new notification observer.

    :param stop: Event object to signal when to stop observing.
    :param result: Event object to signal when expected notifications have
                   been observed.
    :param expected_count: Number of notifications to wait for before
                           signalling.
    :param timeout: How long to wait for notifications before timing out.

    """
    observer = NotificationObserver(stop, result, notifications,
                                    expected_count, timeout)
    observer.start()


def launch_notification_observer(result, notifications, expected_count=0,
                                 timeout=60):
    """
    Launch new thread to listen for when new messages are received.

    :param result: Event object to indicate when required number of messages
                   has been received.
    :param expected_count: Number of messages to wait for before signalling.
    :return: StoppingThread object which should be stopped by caller when
             required.

    """
    launch_thread = StoppingThread(
        target=_start_observer,
        args=[result, notifications, expected_count, timeout])
    return launch_thread
