# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import logging
import re
import subprocess

from ubuntu_system_tests.helpers import wait_until

logger = logging.getLogger(__name__)

OFONO = 'org.ofono'
SIM_MANAGER = 'org.ofono.SimManager'
VOICECALL = 'org.ofono.VoiceCall'
VOICECALL_MANAGER = 'org.ofono.VoiceCallManager'
NETWORK_REGISTER_TIMEOUT = 180


def _get_sim_manager(sim_number):
    bus = dbus.SystemBus()
    modem = bus.get_object('org.ofono', '/ril_{}'.format(sim_number))
    return dbus.Interface(modem, SIM_MANAGER)


def is_pin_locked(sim_number):
    sim_manager = _get_sim_manager(sim_number)
    locked_pins = sim_manager.GetProperties()['LockedPins']
    return 'pin' in locked_pins


def unlock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if is_pin_locked(sim_number):
        sim_manager.UnlockPin('pin', pin)


def lock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if not is_pin_locked(sim_number):
        sim_manager.LockPin('pin', pin)


def _check_modem(ril_modem, properties, modem_number=None,
                 ignore_phonesim=True):
    """Decide if the specified modem should be checked.

    Returns True if this modem should be checked, False if not.

    :param ignore_phonesim: If true then phonesim modems will be ignored.
    :param properties: Properties from the modem
    :param ril_modem: Name of the modem
    :parm modem_number: Number of specific modem to check, or None to check all
    :returns: True if this modem should be checked, False if not.
    """
    PHONESIM_SERIAL = '000000000000000'
    if (str(ril_modem).endswith(str(modem_number)) or modem_number is None):
        if 'Serial' not in properties:
            # This is not a real modem, ignore it
            check_status = False
        elif ignore_phonesim and properties['Serial'] == PHONESIM_SERIAL:
            # This is a phonesim modem and should be ignored
            check_status = False
        else:
            # This modem should be checked
            check_status = True
    else:
        check_status = False
    return check_status


def _get_all_modems(bus):
    """
    Get the DBus interfaces representing each modem
    the device has.

    :returns: A list of modems
    """
    manager = dbus.Interface(
        bus.get_object('org.ofono', '/'), 'org.ofono.Manager')
    return manager.GetModems()


def get_number_of_modems():
    """Return the total number of hardware modems in the device."""
    bus = dbus.SystemBus()
    modems = _get_all_modems(bus)
    return len(modems)


def _sim_present(bus, ril_modem, properties,
                 sim_number=None, ignore_phonesim=True):
    if _check_modem(ril_modem, properties, modem_number=sim_number,
                    ignore_phonesim=ignore_phonesim):
        modem = bus.get_object('org.ofono', ril_modem)
        sim_manager = dbus.Interface(modem, SIM_MANAGER)
        try:
            if sim_manager.GetProperties()['Present']:
                return True
        except dbus.exceptions.DBusException as e:
            logger.warning('exception: {0}detected on modem: {1}'.format(
                           e, ril_modem))
    return False


def is_sim_inserted(sim_number=None, ignore_phonesim=True):
    """Check a SIM is inserted.

    Checks if a SIM card is inserted in the specified
    slot on the device. If no slot is specified then
    all slots will be checked.

    :param ignore_phonesim: If true then phonesim modems will be ignored.
    :param sim_number: The slot index to check for SIM presence

    :returns: True if a SIM is inserted in the specified slot or
    if no slot is specified then if a SIM is inserted in
    any slot, otherwise False
    """
    bus = dbus.SystemBus()
    for ril_modem, properties in _get_all_modems(bus):
        if _sim_present(bus, ril_modem, properties,
                        sim_number, ignore_phonesim):
            return True
    return False


def number_of_sims(ignore_phonesim=True):
    """
    Provide the number of SIMs inserted in the device.

    :returns: The number of SIMs found
    """
    number_of_sims = 0
    bus = dbus.SystemBus()
    for ril_modem, properties in _get_all_modems(bus):
        if _sim_present(bus, ril_modem, properties,
                        ignore_phonesim=ignore_phonesim):
            number_of_sims += 1
    return number_of_sims


def get_sim_name(sim_slot_index=0):
    """
    Provide the network name of the SIM inserted in the device.

    :param sim_slot_index: the index number of the SIM slot
    :return: The network name of the SIM
    """
    account_names = _get_ofono_account_names()
    for account_name in account_names:
        if _is_account_associated(account_name, sim_slot_index):
            return _get_network_name(account_name)


def _get_network_registration_interface(bus, modem):
    """
    Return NetworkRegistration interface for specified modem.

    :param bus: DBus System bus object.
    :param modem: Path of modem to check
    :return: NetworkRegistration interface object.

    """
    modem = bus.get_object('org.ofono', modem)
    return dbus.Interface(modem, 'org.ofono.NetworkRegistration')


def _get_registered_status_for_modem(bus, ril_modem):
    """
    Return Boolean network registration status for specified modem.

    :param bus: DBus System bus object.
    :param modem: Path of modem to check
    :return: True if registered, False otherwise.

    """
    try:
        interface = _get_network_registration_interface(bus, ril_modem)
        if interface.GetProperties()['Status'] == 'registered':
            return True
    except dbus.exceptions.DBusException:
        pass
    return False


def is_sim_registered(sim_number):
    """
    Return True if the specified SIM is registered on a mobile network.

    :param sim_number: String number for the required SIM. Either '0', '1'.
    :return: True if registered on network, False otherwise.

    """
    bus = dbus.SystemBus()
    ril_modem = '/ril_{}'.format(sim_number)
    return _get_registered_status_for_modem(bus, ril_modem)


def is_any_sim_registered():
    """Return True if any available SIM is registered on a mobile network."""
    if is_sim_inserted():
        bus = dbus.SystemBus()
        for ril_modem, properties in _get_all_modems(bus):
            if _get_registered_status_for_modem(bus, ril_modem):
                return True
    return False


def are_all_sims_registered():
    """Return True if all available SIMs are registered on a mobile network."""
    if is_sim_inserted():
        bus = dbus.SystemBus()
        for ril_modem, properties in _get_all_modems(bus):
            if (_sim_present(bus, ril_modem, properties) and
               not _get_registered_status_for_modem(bus, ril_modem)):
                return False
        return True
    return False


def wait_for_sim_registered(sim, timeout=NETWORK_REGISTER_TIMEOUT):
    """
    Wait until the specified SIM is registered on the network.

    :param sim: String number for the required SIM. Either '0', '1'.
    :param timeout: How long to wait before timing out.
    :return: True if registered on network, False otherwise.

    """
    return wait_until(is_sim_registered, sim, timeout=timeout)


def wait_for_any_sim_registered(timeout=NETWORK_REGISTER_TIMEOUT):
    """
    Wait until any available SIM is registered on the network.

    :param timeout: How long to wait before timing out.
    :return: True if any SIM registered on network, False otherwise.

    """
    return wait_until(is_any_sim_registered, timeout=timeout)


def wait_for_all_sims_registered(timeout=NETWORK_REGISTER_TIMEOUT):
    """
    Wait until all available SIMs are registered on the network.

    :param timeout: How long to wait before timing out.
    :return: True if all available SIMs registered on network, False otherwise.

    """
    return wait_until(are_all_sims_registered, timeout=timeout)


def _get_ofono_account_names():
    """Returns a list, containing low-level names of ofono accounts."""
    names = subprocess.check_output(['mc-tool', 'list'],
                                    universal_newlines=True)
    return [name for name in names.splitlines()]


def _is_account_associated(account_name, slot_index):
    """
    Check if the given ofono account is associated with the
    given SIM slot.

    :param account_name: ofono account name
    :param slot_index: Index number of the SIM slot
    :return: True if the given account is associated with the
    given SIM card slot, otherwise False
    """
    account_details = _get_details_for_account(account_name)
    for line in account_details:
        if '/ril_{}'.format(slot_index) in line:
            return True
    return False


def _get_network_name(account_name):
    """
    Provide cellular provider name of the given account
    :param account_name: ofono account name
    :return: The name of the cellular network associated with the given
    ofono account
    """
    account_details = _get_details_for_account(account_name)
    for line in account_details:
        if 'Current:' in line:
            return re.findall(r'"([^"]*)"', line)[0]


def _get_details_for_account(account_name):
    """
    Provide all available details about a specific ofono account.
    :param account_name: ofono account name
    :return: A list of characteristics and properties of the
    relevant ofono account
    """
    details = subprocess.check_output(['mc-tool', 'show', account_name],
                                      universal_newlines=True)
    return [line for line in details.splitlines()]


def _is_modem_in_state(bus, path, properties, state):
    """Return True if any call on specified modem is in required state, False
    otherwise.
    :param bus: DBus system bus.
    :param path: Modem path.
    :param properties: Modem properties.
    :param state: State of call to check for.
    """
    if VOICECALL_MANAGER in properties['Interfaces']:
        mgr = dbus.Interface(bus.get_object(OFONO, path), VOICECALL_MANAGER)
        for call_path, properties in mgr.GetCalls():
            if properties['State'] == state:
                return True
    return False


def _is_call_in_state(state, sim=None):
    """Return True if a call is in specified state, False if not.
    :param state: State to check for.
    :param sim: Sim number to check, either '0' or '1'. Only calls on this SIM
                will be checked. If not specified then all SIMs will be
                checked.
    """
    bus = dbus.SystemBus()
    for path, properties in _get_all_modems(bus):
        if sim and not path.endswith(sim):
            # sim number is specified and this is not it
            continue
        if _is_modem_in_state(bus, path, properties, state):
            return True
    return False


def is_call_on_hold(sim=None):
    """ Return True if a call is in on hold, False if not.
    :param sim: Sim number to check, either '0' or '1'. Only calls on this SIM
                will be checked. If not specified then all SIMs will be
                checked.
    """
    return _is_call_in_state(state='held', sim=sim)


def is_call_active(sim=None):
    """ Return True if a call is in progress, False if not.
    :param sim: Sim number to check, either '0' or '1'. Only calls on this SIM
                will be checked. If not specified then all SIMs will be
                checked.
    """
    return _is_call_in_state(state='active', sim=sim)


def is_call_ringing(sim=None):
    """ Return True if a call is ringing, False if not.
    :param sim: Sim number to check, either '0' or '1'. Only calls on this SIM
                will be checked. If not specified then all SIMs will be
                checked.
    """
    return _is_call_in_state(state='incoming', sim=sim)


def end_call():
    """Programmatically end all ongoing calls."""
    bus = dbus.SystemBus()
    for path, properties in _get_all_modems(bus):
        if VOICECALL_MANAGER in properties['Interfaces']:
            mgr = dbus.Interface(bus.get_object(OFONO, path),
                                 VOICECALL_MANAGER)
            for call_path, properties in mgr.GetCalls():
                try:
                    call = dbus.Interface(bus.get_object(OFONO, call_path),
                                          VOICECALL)
                    call.Hangup()
                except dbus.exceptions.DBusException:
                    # call is already ended
                    pass


def get_modem_for_data():
    bus = dbus.SystemBus()
    modems = _get_all_modems(bus)
    for modem in modems:
        modem_number = int(str(modem[0])[-1])
        if is_sim_inserted(modem_number):
            connman = dbus.Interface(bus.get_object('org.ofono', modem[0]),
                                     'org.ofono.ConnectionManager')
            properties = connman.GetProperties()
            if properties.get("Powered"):
                return modem_number
        else:
            continue
    return None


def set_modem_for_data(modem_number):
    bus = dbus.SystemBus()
    modems = _get_all_modems(bus)
    for modem in modems:
        connman = dbus.Interface(bus.get_object('org.ofono', modem[0]),
                                 'org.ofono.ConnectionManager')
        if is_sim_inserted(int(str(modem[0])[-1])):
            if modem_number and str(modem[0])[-1] == modem_number:
                connman.SetProperty("Powered", dbus.Boolean(1))
            else:
                connman.SetProperty("Powered", dbus.Boolean(0))


def get_imei_list():
    """ Return a list with the imei for the different modems """

    bus = dbus.SystemBus()
    imei_list = []
    for ril_modem, properties in _get_all_modems(bus):
        if 'Serial' in properties:
            imei_list.append(properties['Serial'])
    return imei_list
