# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import datetime
import lsb_release
import os
import re
import subprocess
import uuid

from autopilot.platform import model
from systemimage import config as sys_info

from ubuntu_system_tests.helpers import click


def get_device_name():
    """
    Return the device's model name.

    If this is run on desktop, it will return 'desktop'.

    """
    return sys_info.config.device or model().lower()


def get_channel_name():
    """ Return the build channel name. In case there is not channel defined, it
    return the default value
    """
    default = 'ubuntu-touch/rc-proposed/bq-aquaris.en'
    return _get_image_info('service', 'channel', default)


def get_build_number():
    """ Return the build number. In case there is not channel defined, it
    return the default value
    """
    return _get_image_info('service', 'build_number', '0')


def _get_image_info(category, item, default):
    """ Return the image item requested. In case it is not defined, it return
    the default value
    """
    file = '/etc/system-image/channel.ini'
    if not os.path.isfile(file):
        return default
    parser = configparser.ConfigParser()
    parser.read(file)
    return parser[category][item]


def is_agps_supported():
    """Return True if a agps is supported, False if it is not."""
    agps_supported = True
    try:
        subprocess.check_call(
            ['dpkg-query --status ubuntu-location-provider-here'],
            shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        agps_supported = False
    return agps_supported


def is_camera_supported():
    """Return True if a camera is supported, False if it is not."""
    return click.is_package_installed('com.ubuntu.camera')


def is_wifi_supported():
    """Return True if wifi is supported, False if it is not."""
    wifi_supported = True
    try:
        subprocess.check_call(
            ['nmcli -t -f type device | grep wifi'],
            shell=True, stdout=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        wifi_supported = False
    return wifi_supported


def get_mac_address():
    """ Retrieve the mac address for the wifi interface """
    return ':'.join(re.findall('..', '%012x' % uuid.getnode()))


def get_bluetooth_address():
    return subprocess.check_output('hcitool dev',
                                   shell=True,
                                   universal_newlines=True).split()[-1]


def get_os_and_version():
    """ Retrieve a string with the os and version installed """
    return lsb_release.get_lsb_information()['DESCRIPTION']


def get_os_last_update_date():
    """ Retrieve a datetime according to the os last update  """
    path = '/userdata/.last_update'
    if not os.path.exists(path):
        path = '/home'

    return datetime.datetime.fromtimestamp(os.path.getctime(path))


def _get_build_prop(prop):
    """ Retrieve the build property using the getprop command """
    return subprocess.check_output('getprop {}'.format(prop),
                                   shell=True,
                                   universal_newlines=True).split('\n')[0]


def get_serial_number():
    return _get_build_prop('ro.serialno')


def get_device_manufacturer():
    return _get_build_prop('ro.product.manufacturer')


def get_device_model():
    return _get_build_prop('ro.product.model')
