# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import subprocess
import shlex
import stat

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from ubuntu_system_tests.helpers.processes import get_process_id
from ubuntu_system_tests.helpers.testbed import write_to_file_with_sudo


def get_auto_brightness_current_setting():
    """Return current auto brightness state from gsettings."""
    command = 'gsettings get com.ubuntu.touch.system auto-brightness'
    return subprocess.check_output(shlex.split(command),
                                   universal_newlines=True).strip()


def set_auto_brightness_setting(value):
    """Change auto brightness from gsettings."""
    command = 'gsettings set com.ubuntu.touch.system ' \
              'auto-brightness {}'.format(value)
    return subprocess.check_call(shlex.split(command))


def set_brightness_setting(value):
    """Set brightness through powerd, value should be between 10 and 255."""
    command = 'powerd-cli brightness {}'.format(value)
    subprocess.check_call(shlex.split(command),
                          stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)


def get_brightness_setting():
    """Return the current brightness setting value from gsettings."""
    command = 'gsettings get com.ubuntu.touch.system brightness'
    return subprocess.check_output(shlex.split(command),
                                   universal_newlines=True).strip()


def get_powerd_fifo_path():
    """Return path of fake sensor fifo used by powerd process."""
    return '/tmp/sensor-fifo-{}'.format(get_process_id('powerd'))


def write_to_powerd_fifo(commands):
    """
    Write specified commands to the powerd sensor fifo buffer.
    :param commands: List of commands to write.
    """
    for command in commands:
        echo_cmd = 'echo "{}"'.format(command)
        write_to_file_with_sudo(get_powerd_fifo_path(), echo_cmd, append=True)


def validate_powerd_fifo_ready():
    """Wait until the powerd fifo buffer is ready and validate."""
    fifo_path = get_powerd_fifo_path()
    Eventually(Equals(True)).match(
        lambda: os.path.exists(fifo_path))
    assert(stat.S_ISFIFO(os.stat(fifo_path).st_mode))


def setup_powerd_sensors():
    """Initialise the fake sensors used in the powerd process."""
    cmds = ['create proximity',
            'create accel 0 1000 0.1',
            'create light 0 10 1',
            '10 proximity far']
    return write_to_powerd_fifo(cmds)
