# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase

from ubuntu_system_tests.helpers.autopilot import validate_dbus_object
from ubuntu_system_tests.helpers.unity8 import UNITY8_DASH_PATH_ROOT


class ScopesList(UbuntuUIToolkitCustomProxyObjectBase):
    """
    The 'Manage' page for managing scopes.

    Owned by the dash, retrieved by unity8.dash.open_manage_scopes().

    """

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_DASH_PATH_ROOT, b'ScopesList',
            objectName='scopesList')

    def _is_list_item_visible(self, list_item):
        """Return the visibility of the list item on the screen.

        Is list item origin within screen bounds?  (Doesn't factor
        partial visibility, etc.)

        """
        y = list_item.globalRect[1]
        return y >= 0 and y <= self.height

    def _get_qquickflickable(self):
        """Return the scrollable/flickable for the scopes list."""
        return self.get_children_by_type('QQuickFlickable')[0]

    def _get_scopes_list_by_category(self, category_name):
        """Return ScopesListCategoryItems under the given category name.

        :param category_name: probably 'favorites' or 'other'

        """
        return self.wait_select_single(
            'ScopesListCategory',
            objectName='scopesListCategory{}'.format(category_name))

    def get_scopes_list(self):
        """Return a list of all scopes."""
        return self.select_many('ScopesListCategoryItem')

    def get_favorite_scopes_list(self):
        """Return a list of scopes under the 'Home' section."""
        scopes_list = self._get_scopes_list_by_category('favorites')
        return scopes_list.select_many('ScopesListCategoryItem')

    def get_other_scopes_list(self):
        """Return a list of scopes under the 'Also installed' section."""
        scopes_list = self._get_scopes_list_by_category('other')
        return scopes_list.select_many('ScopesListCategoryItem')

    def favorite_scopes_are_visible(self):
        """Validate that all favorite scopes are visible."""
        favorite_scopes = self.get_favorite_scopes_list()
        for scope in favorite_scopes:
            # weird 'dragItem' interferes; only consider items with text
            if scope.text == '':
                continue
            if self._is_list_item_visible(scope) is False:
                return False
        return True

    def other_scopes_are_visible(self):
        """Validate that at least one 'other' scope is visible."""
        return len(self.get_other_scopes_list()) > 0

    def scrolling_down_shows_more_scopes(self):
        """Scroll down, validate that more scopes are shown.

        The list of scopes changes when we scroll, i.e. the list
        visible at the bottom is different from that at the top.  For
        a small list of scopes we expect this to be false :) .

        """
        scopes_top = set(
            [scope.text for scope in
             filter(self._is_list_item_visible,
                    self.get_scopes_list())])
        flickable = self._get_qquickflickable()
        flickable.swipe_to_bottom()
        scopes_bottom = set(
            [scope.text for scope in
             filter(self._is_list_item_visible,
                    self.get_scopes_list())])
        return scopes_top != scopes_bottom
