# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from autopilot import exceptions
from collections import namedtuple
from unity8 import dash as unity8_dash
from ubuntu_system_tests.helpers.autopilot.list import order_by_y_coord
from ubuntu_system_tests.helpers.ubuntuuitoolkit.pageheader import (
    DashPageHeader)
from ubuntu_system_tests.helpers.unity8 import get_dash
from ubuntu_system_tests.helpers.url_dispatcher import go_to_url

ScopeDataItem = namedtuple('ScopeDataItem', ['data', 'globalRect'])

IMAGE_TYPES = ['QQuickImage', 'CroppedImageMinimumSourceSize']

logger = logging.getLogger(__name__)


class GenericScopeView(unity8_dash.GenericScopeView):
    """
    This class is a base class for all scope CPOs and should never be
    instantiated directly. This avoids autopilot CPO errors.

    """

    @classmethod
    def validate_dbus_object(cls, path, state):
        """This class is a base class and should never be selected directly."""
        return False

    def swipe_to_top(self):
        """Swipe to the top of the scope."""
        list_view = self.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')
        list_view.swipe_to_top()

    def click_scope_item(self, category, title, press_duration=0.10):
        """Override parent implementation making it click at the top of the
        application button to avoid issue when the button is not displayed
        completely

        :parameter category: The name of the category where the item is.
        :parameter title: The title of the item.

        """
        category_element = self._get_category_element(category)
        icon = category_element.wait_select_single(title=title)
        list_view = self.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')
        list_view.swipe_child_into_view(icon)

        x, y, width, height = icon.globalRect
        self.pointing_device.move(x + 1, y + 1)
        self.pointing_device.click(press_duration=press_duration)

    def _get_data_from_object(self, object):
        """
        Extract the required data from object.

        For 'Label' object this is 'text' property.
        For 'QQuickImage' object this is 'source' property.

        :param object: The object to examine
        :return: The extracted data

        """
        if 'text' in object.get_properties():
            data = object.text
        else:
            data = object.source
        return data

    def _add_sorted_new_items_to_existing(self, new_items, existing_items):
        """
        Sort the new items in order of y-coord and add to
        existing_items list.

        """
        for item in order_by_y_coord(new_items, include_off_screen=True):
            existing_items.append(item.data)
        return existing_items

    def _get_content_of_types(self, types, existing_items):
        """
        Extract the required types from the scope. Add all new items to the
        existing_items list and return it.

        :param types: A list of object types to look for.
        :param existing_items: A list of ScopeDataItems. This represents any
                              items that have already been found. No duplicate
                              items will be added to the list.
        :return: List of ScopeDataItems including any new items found.

        """
        # find any new items not already in existing_items
        new_items = []
        for type in types:
            for item in self.select_many(type, visible=True):
                item_data = self._get_data_from_object(item)
                if item_data and item_data not in existing_items:
                    new_items.append(ScopeDataItem(item_data, item.globalRect))

        return self._add_sorted_new_items_to_existing(
            new_items, existing_items)

    def _get_content_labels(self, existing_items, include_images):
        """Return list of data items from all Label items displayed."""
        types = ['UCLabel']
        if include_images:
            types.extend(IMAGE_TYPES)
        return self._get_content_of_types(types, existing_items)

    def _get_content_headers(self, existing_items):
        """Return list of data items from all Header items displayed."""
        return self._get_content_of_types(['Header'], existing_items)

    def _get_listview(self):
        return self.select_single(
            'ListViewWithPageHeader', objectName='categoryListView')

    def get_all_content(self, include_images, scroll_to_end=True):
        """
        Return all data content from the current scope, including Headers,
        Labels and optionally QQuickImages. The data items returned will be
        orderd according to the order that they appear on the screen
        veritcally.

        :param include_images: Whether to include image sources in the results
        :param scroll_to_end: Whether to scroll to the end of the scope to
                              find the results.
        :return: List of data items ordered by y co-ordinate.

        """
        list_view = self._get_listview()
        all_headers = []
        all_labels = []
        at_bottom = False
        while not at_bottom:
            all_headers = self._get_content_headers(all_headers)
            all_labels = self._get_content_labels(all_labels, include_images)
            if not list_view.atYEnd and scroll_to_end:
                list_view.swipe_to_show_more_below()
            else:
                at_bottom = True
        self.swipe_quickly_to_top()
        return all_headers, all_labels

    def get_title(self):
        """Return title of current scope."""
        return self.select_single(objectName="scopePageHeader").title

    def get_url(self):
        """Return scope url that can be used with url-dispatcher."""
        return 'scope://{name}'.format(name=self.objectName)

    def refresh(self):
        """Refresh scope content and wait until it is ready."""
        go_to_url(self.get_url())
        self.wait_for_processing_to_complete()

    def swipe_quickly_to_top(self):
        """Swipe the scope quickly to the top."""
        list_view = self._get_listview()
        start_x = list_view.globalRect.x + (list_view.globalRect.width // 2)
        stop_x = start_x
        from ubuntuuitoolkit._custom_proxy_objects import _flickable
        containers = list_view._get_containers()
        start_y = _flickable._get_visible_container_top(containers) + 5
        stop_y = _flickable._get_visible_container_bottom(containers) - 50
        while not list_view.atYBeginning:
            list_view._slow_drag(start_x, stop_x, start_y, stop_y, rate=20)
            list_view.dragging.wait_for(False)
            list_view.moving.wait_for(False)

    def _get_setup_skip_button(self):
        try:
            section = self.select_single(
                'CardVerticalJournal', objectName='skip')
            return section.select_single(objectName='delegate0', visible=True)
        except exceptions.StateNotFoundError:
            return None

    def _press_skip_setup_button(self):
        """Press the skip setup button and wait for the header animation to
        stop moving.
        """
        self.pointing_device.click_object(self._get_setup_skip_button())
        self.wait_for_header_to_stop_moving()

    def _get_first_visible_object(self, objects):
        """ Retrieve the object that is currently being displayed based on its
        position
        :param objects: the list of objects to compare
        :return: First visible item from list

        """
        return order_by_y_coord(objects)[0]

    def has_skip_button(self):
        """ Indicates if the skip button is present"""
        return self._get_setup_skip_button() is not None

    def skip_setup(self):
        """
        Skip the accounts setup by clicking the button 'Skip & set up later'.
        This method should be used just for the scopes: Today and Photos
        """
        self.scroll_to_bottom()
        self._press_skip_setup_button()
        get_dash().get_current_page_header().go_back()

    def scroll_to_bottom(self):
        """ Scroll to the bottom of the scope """
        list_view = self._get_listview()
        list_view.swipe_to_bottom()

    def get_search_button(self):
        return self.select_single(objectName='search_button')

    def get_search_text_field(self):
        return self.select_single(objectName='searchTextField')

    def search(self, text):
        """ Search for a specific text in the scope and waits until the results
        are displayed
        :param text: The text to search
        """
        self.pointing_device.click_object(self.get_search_button())
        self.wait_for_header_to_stop_moving()

        self.get_search_text_field().write(text)
        self.wait_for_processing_to_start(silent_fail=True)
        self.wait_for_processing_to_complete()

    def _get_scope_page_header(self):
        return self.select_single(
            DashPageHeader, objectName='scopePageHeader')

    def wait_for_header_to_stop_moving(self):
        self._get_scope_page_header().wait_until_not_moving()

    def wait_for_processing_to_start(self, timeout=10, silent_fail=False):
        """Wait for the processing to start. In some cases the processing state
        can be very brief which means its not always reliable to match the
        required state. For this reason it is possible to fail silently, in
        which case an exception will not be raised and the function will only
        wait for the timeout period to expire.
        :param timeout: How long to wait.
        :param silent_fail: If True then don't raise an exception if processing
                            doesn't begin within timeout period.
        """
        try:
            self.processing.wait_for(True, timeout=timeout)
        except AssertionError as e:
            if not silent_fail:
                raise(e)

    def wait_for_processing_to_complete(self):
        """Wait for the processing indicator to disappear."""
        self.processing.wait_for(False, timeout=240)
