# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
from subprocess import (
    CalledProcessError,
    check_call,
    check_output,
)

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers.scopes.apps import (
    launch_application_from_apps_scope
)
from ubuntu_system_tests.helpers.unity8 import close_app

APP = 'system-settings'
APP_NAME = 'System Settings'
APP_WIN_ID = 'ubuntu-system-settings'

SIM_1 = '1'
SIM_2 = '2'
SIM_ASK = 'ask'


def launch_system_settings():
    """
    Launch system settings from apps scope and return proxy object.

    :return: Proxy object for system settings.

    """
    launch_application_from_apps_scope(APP_NAME)
    return get_system_settings_proxy()


def close_system_settings():
    """ Close the system settings app using task switcher """
    if is_system_settings_running():
        close_app(APP_WIN_ID)


def is_system_settings_running():
    """ Indicate if the system settings is currently running """
    return processes.is_process_running(APP)


def get_system_settings_proxy():
    """
    Return system settings proxy object from existing process.

    :return: Proxy object for system settings.

    """
    from ubuntu_system_tests.helpers.system_settings import _cpo  # NOQA
    proxy = autopilot.get_proxy_object_for_existing_app(
        APP, cleanup_process=APP_WIN_ID)
    return proxy.main_view


def get_ril_modem_for_sim(sim):
    """
    Convert the sim slot number to ril modem id, /ril_0 or /ril_1.
    :param sim: '1' or '2'
    :return: ril modem id, '/ril_0' or '/ril_1'

    """
    ril_index = str(int(sim) - 1)
    return '/ril_{}'.format(ril_index)


def get_default_sim_for_messages():
    """Return default sim for messages, stripped of quotation marks."""
    return check_output(
        ['gsettings', 'get', 'com.ubuntu.phone',
         'default-sim-for-messages'],
        universal_newlines=True).strip().strip("'")


def set_default_sim_for_messages(sim):
    """
    Set the default sim for messages based on sim slot number.
    :param sim: '1', '2', '/ril_0', '/ril_1' or 'ask'

    """
    if sim in (SIM_1, SIM_2):
        sim = get_ril_modem_for_sim(sim)
    set_default_modem_for_messages(sim)


def set_default_modem_for_messages(modem):
    """
    Set the default modem for messages based on modem.
    :param modem: '/ril_0', '/ril_1' or 'ask'

    """
    return check_output(
        ['gsettings', 'set', 'com.ubuntu.phone',
         'default-sim-for-messages', modem],
        universal_newlines=True)


def get_default_sim_for_calls():
    """Return default sim for messages, stripped of quotation marks."""
    return check_output(
        ['gsettings', 'get', 'com.ubuntu.phone',
         'default-sim-for-calls'],
        universal_newlines=True).strip().strip("'")


def set_default_sim_for_calls(sim):
    """
    Set the default sim for messages based on sim slot number.
    :param sim: '1', '2', '/ril_0', '/ril_1' or 'ask'

    """
    if sim in (SIM_1, SIM_2):
        sim = get_ril_modem_for_sim(sim)
    set_default_modem_for_calls(sim)


def set_default_modem_for_calls(modem):
    """
    Set the default modem for messages based on modem.
    :param modem: '/ril_0', '/ril_1' or 'ask'

    """
    return check_output(
        ['gsettings', 'set', 'com.ubuntu.phone',
         'default-sim-for-calls', modem],
        universal_newlines=True)


def ensure_settings_page_visible(settings_app_proxy, page_object_name):
    """Wait until the requested settings page is visible."""
    page = settings_app_proxy.wait_select_single(objectName=page_object_name)
    page.active.wait_for(True)


def ensure_power_settings_page_visible(settings_app_proxy):
    """Wait until battery settings page is visible."""
    page = settings_app_proxy.wait_select_single('PageComponent',
                                                 title='Battery')
    page.active.wait_for(True)


def get_rotation_lock():
    """Return current value for rotation lock."""
    return check_output(
        ['gsettings', 'get',
         'com.ubuntu.touch.system', 'rotation-lock'],
        universal_newlines=True)


def set_rotation_lock(lock):
    """Set the value for the rotation lock setting."""
    try:
        check_call(
            ['gsettings', 'set', 'com.ubuntu.touch.system',
             'rotation-lock', lock],
            universal_newlines=True)
    except CalledProcessError:
        return False
    return True


def _get_sound_interface():
    """Return sound interface dbus object."""
    bus = dbus.SessionBus()
    return dbus.Interface(
        bus.get_object('com.canonical.indicator.sound',
                       '/com/canonical/indicator/sound'), 'org.gtk.Actions')


def get_silent_mode():
    """Return current boolean value for silent mode setting."""
    return bool(_get_sound_interface().Describe('silent-mode')[2][0])


def set_silent_mode(state):
    """Set boolean value for silent mode setting."""
    _get_sound_interface().SetState('silent-mode', state, [])
