# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import functools
import logging
import os
import requests

from requests.auth import HTTPBasicAuth

from ubuntu_system_tests.helpers.messaging import (
    calculate_sent_message_count, split_into_received_messages)
from ubuntu_system_tests.helpers import web

logger = logging.getLogger(__name__)

TWILIO_API_BASE = "https://api.twilio.com"
TWILIO_API_URL = TWILIO_API_BASE + "/2010-04-01"

LOG_MESSAGE_NOT_FOUND = ('ERROR: Message not found with length {l} and '
                         'body:\n{b}')


class Telephony:

    def __init__(self, credentials, backend='twilio'):
        self._backend = _get_backend(backend)(credentials)

    def send_sms(self, to, from_, message):
        return self._backend.send_sms(to, from_, message)

    def get_sent_sms(self, to, from_, date_sent, body):
        return self._backend.get_sent_sms(to, from_, date_sent, body)

    def send_mms(self, to, from_, message, media):
        return self._backend.send_mms(to, from_, message, media)

    def make_phonecall(self, to, from_, hangup_timeout, call_url):
        return self._backend.make_phonecall(
            to, from_, hangup_timeout, call_url)

    def get_call(self, sid):
        return self._backend.get_call(sid)

    def get_calls(self, from_, to, time_from, time_to=None):
        return self._backend.get_calls(from_, to, time_from, time_to)

    def is_call_in_progress(self, from_, to):
        return self._backend.is_call_in_progress(from_, to)

    def is_call_ringing(self, from_, to):
        return self._backend.is_call_ringing(from_, to)

    def is_call_active(self, from_, to):
        return self._backend.is_call_active(from_, to)

    def download_mms_content(self, to, from_, date_sent, body):
        return self._backend.download_mms_content(to, from_, date_sent, body)

    def get_properties(self):
        return self._backend.get_properties()

    def hangup_call(self, call_id):
        return self._backend.hangup_call(call_id)

    def cancel_call(self, call_id):
        return self._backend.cancel_call(call_id)


NeedBackendError = functools.partial(
    NotImplementedError,
    "This class cannot be instantiated. Please use a specific backend."
)


class TelephonyBackend:

    def __init__(self, credentials):
        """
        :param credentials: dictionary containing the required credential
          details specific for the backend.

        :raises ValueError: if any required credential details are missing.

        """
        raise NeedBackendError()

    def send_sms(self, to, from_, message):
        raise NeedBackendError()

    def get_sent_sms(self, to, from_, date_sent, body):
        raise NeedBackendError()

    def send_mms(self, to, from_, message, media):
        raise NeedBackendError()

    def make_phonecall(self, to, from_, hangup_timeout, call_url):
        raise NeedBackendError()

    def get_call(self, sid):
        raise NeedBackendError()

    def get_calls(self, from_, to, time_from, time_to=None):
        raise NeedBackendError()

    def is_call_in_progress(self, from_, to):
        raise NeedBackendError()

    def is_call_ringing(self, from_, to):
        raise NeedBackendError()

    def is_call_active(self, from_, to):
        raise NeedBackendError()

    def get_properties(self):
        raise NeedBackendError()

    def hangup_call(self, call_id):
        raise NeedBackendError()

    def cancel_call(self, call_id):
        raise NeedBackendError()


class TwilioBackend(TelephonyBackend):
    """Backend that utilises the Twilio service.

    Required credentials are:
      - AccountSid: This represents the account id
      - AuthToken: This is the token used to authenticate the request against
        the account (Note: It must be kept secret!).

    """

    def __init__(self, credentials):
        _raise_if_required_credentials_missing(
            credentials, 'AccountSid', 'AuthToken')
        self._account_sid = credentials['AccountSid']
        self._account_auth = credentials['AuthToken']
        self._http_auth = HTTPBasicAuth(self._account_sid, self._account_auth)

    def get_properties(self):
        """Return dictionary of properties for this telephony provider."""
        properties = {'sms-concatenation': True}
        return properties

    def send_sms(self, to, from_, message):
        return self._send_message(to, from_, message)

    def send_mms(self, to, from_, message, media_url):
        return self._send_message(to, from_, message, media_url)

    def _send_message(self, to, from_, message, media_url=None):
        """
        Send an SMS or MMS message. If no media_url parameter is specified then
        the message will be sent as an SMS message. If a media_url parameter is
        specified then the content will be downloaded then attached to the
        message and sent as an MMS message.

        :param to: Phone number of recipient
        :param from_: Phone number of sender
        :param message: Body of the message to send
        :param media_url: Url of file to attach to message
        :return: True if message sent successfully, False otherwise
        """
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Messages')
        params = {'From': from_, 'To': to, 'Body': message}
        if media_url:
            params['MediaUrl'] = media_url

        response = requests.post(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        return response.json()

    def get_sent_sms(self, to, from_, date_sent, body):
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Messages')
        params = {'From': from_,
                  'To': to,
                  'DateSent': date_sent.strftime('%Y-%m-%d'),
                  'num_segments': str(calculate_sent_message_count(body))}
        response = requests.get(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        return self.get_sent_messages(response, body)

    def get_sent_messages(self, response, body):
        """
        Find all matching messages that were sent to telephony service.
        This includes breaking down the sent message into chunks of maximum
        message size and checking that they are all received by the telephony
        service.

        :param response: JSON response from the telephony service
        :param body: The message body that is expected to be received
        :return: List of messages matching the required messages

        """
        concatenate = self.get_properties()['sms-concatenation']
        msg_list = []
        for sent_msg in split_into_received_messages(body, concatenate):
            msg = self._find_message_in_response(response, sent_msg)
            if msg:
                msg_list.append(msg)
            else:
                logger.error(LOG_MESSAGE_NOT_FOUND.format(l=len(sent_msg),
                                                          b=sent_msg))
        return msg_list

    def _find_message_in_response(self, response, message):
        for response_msg in response.json()['messages']:
            if response_msg['body'] == message:
                return response_msg
        return None

    def download_mms_content(self, to, from_, date_sent, body):
        """Download the MMS attachment from twilio and return its path."""
        for message in self.get_sent_sms(to, from_, date_sent, body):
            download_url = self._get_mms_content_url(message)
            if download_url:
                return web.download_file(download_url)
        return None

    def _get_mms_content_url(self, msg):
        """
        Query twilio and get the MMS content url.
        :param msg: the message object to get mms content from
        :return: url to the mms content
        """
        # TODO: Handle multiple MMS attachments. -- om26er 2015-08-09
        media_url = TWILIO_API_BASE + msg.get('subresource_uris').get('media')
        response = requests.get(media_url, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        for media in response.json()['media_list']:
            json_url = TWILIO_API_BASE + media.get('uri')
            # Strip .json from the url that is the actual link with the
            # attachment.
            return os.path.splitext(json_url)[0]
        return None

    def make_phonecall(self, to, from_, hangup_timeout, call_url):
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Calls')
        params = {'From': from_,
                  'To': to,
                  'Url': call_url,
                  'Method': 'GET',
                  'Timeout': hangup_timeout}
        response = requests.post(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        return response.json()

    def get_call(self, call_id):
        """Get a call with specified id."""
        url = self._get_call_url(call_id)
        response = requests.get(url, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        return response.json()

    def get_calls(self, from_, to, time_from, time_to=None):
        """Return a list of calls made between time_from and time_to.
        :param from_: Originating phone number.
        :param to: Terminating phone number.
        :param time_from: Starting time of time range to check.
        :param time_to: Ending time of time range to check. If None then time
                        range will run from time_from until current time.
        :return: List of call dictionary objects.
        """
        log = self._get_call_log(from_, to, time_from, time_to)
        return log['calls']

    def _is_call_in_state(self, from_, to, state):
        """Return True if call currently active between specified numbers is
        in specified state, False otherwise."""
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Calls')
        params = {'From': from_,
                  'To': to,
                  'Status': state}
        response = requests.get(url, params=params, auth=self._http_auth)
        return len(response.json()['calls']) > 0

    def is_call_in_progress(self, from_, to):
        """Return True if call between numbers is in 'in_progress' state."""
        return self._is_call_in_state(from_, to, 'in-progress')

    def is_call_ringing(self, from_, to):
        """Return True if call between numbers is in 'ringing' state."""
        return self._is_call_in_state(from_, to, 'ringing')

    def is_call_active(self, from_, to):
        """Return True if call between numbers is either ringing or
        in-progress."""
        return (self.is_call_ringing(from_, to) or
                self.is_call_in_progress(from_, to))

    def _get_call_log(self, from_, to, time_from, time_to=None):
        """Return call log for calls matching specified criteria.
        :param from_: Originating phone number.
        :param to: Terminating phone number.
        :param time_from: Starting time of time range to check.
        :param time_to: Ending time of time range to check. If None then time
                        range will run from time_from until current time.
        :return: JSON dictionary containing matching call logs.
        """
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Calls')
        params = {'From': from_,
                  'To': to,
                  'StartTime>': time_from}
        if time_to:
            params['EndTime<'] = time_to
        response = requests.get(url, params=params, auth=self._http_auth)
        if response.ok:
            return response.json()
        else:
            logger.error(response.text)

    def _get_call_url(self, call_id):
        """Return API get logged call information to post to."""
        url = TWILIO_API_URL + '/Accounts/{sid}/Calls/{call_id}.json'.format(
            sid=self._account_sid,
            call_id=call_id
        )
        return url

    def _get_api_url(self, action):
        """Return API URL to post to."""
        api_url = TWILIO_API_URL + '/Accounts/{sid}/{action}.json'.format(
            sid=self._account_sid,
            action=action
        )
        return api_url

    def _set_call_params(self, call_id, status):
        """Modify a call's parameters to those specified.
        :param call_id: Id of the call to modify.
        :param status: Status to set call to.
        """
        url = TWILIO_API_URL + '/Accounts/{sid}/Calls/{call_id}'.format(
            sid=self._account_sid,
            call_id=call_id
        )
        params = {'Status': status}
        response = requests.post(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)

    def hangup_call(self, call_id):
        """Hang up a live call."""
        self._set_call_params(call_id, status='completed')

    def cancel_call(self, call_id):
        """Cancel a queued or ringing call."""
        self._set_call_params(call_id, status='canceled')


class FakeBackend(TelephonyBackend):
    """A verified fake for use with testing.

    Does not make any calls to external resources nor does it intiate any
    actual calls/messages.

    """
    pass


def _make_rest_request(url, params, auth_details):
    """Return request object containing the results of the REST request."""
    # i.e. r = requests.post(....)
    # return r or should we check the status code here and raise an exception
    # if it's not what's expected (could be another arg.)
    pass


def _get_backend(backend):
    """Return the requested backend class

    :raises ValueError: if the requested backend was not found.

    """
    # i.e. return TwiloBackend (well, smarter than that)
    # The register (or whatever) could be a lot nicer too.

    backend_register = dict(
        twilio=TwilioBackend,
        fake=FakeBackend,
    )
    return backend_register[backend]


def _raise_if_number_doesnt_look_valid(number):
    """Ensure number looks valid (i.e. E.164 format).

    Does not ensure the number is in service etc.
    """
    pass


def _raise_if_required_credentials_missing(credentials, *required_keys):
    """Raise ValueError if any required keys are absent from credentials.

    The exception string will include a description of which keys are missing.

    """
    missing_keys = []
    for key in required_keys:
        if credentials.get(key, None) is None:
            missing_keys.append(key)

    if missing_keys:
        raise ValueError(
            "Missing required credential detail{plural}: '{details}'".format(
                plural='s' if len(missing_keys) > 1 else '',
                details=', '.join(missing_keys))
        )
