# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import subprocess
import shlex

import fixtures


GSETTINGS_SCHEMA = 'com.canonical.keyboard.maliit'
AUTO_CAPITALIZATION_KEY = 'auto-capitalization'
AUTO_CORRECTION_KEY = 'auto-completion'
WORDS_SUGGESTIONS_KEY = 'predictive-text'


def _is_setting_for_key_enabled(key):
    command = 'gsettings get {schema} {key}'.format(schema=GSETTINGS_SCHEMA,
                                                    key=key)
    output = subprocess.check_output(shlex.split(command),
                                     universal_newlines=True).strip()
    return output == 'true'


def _set_keyboard_setting(key, enable=True):
    command = 'gsettings set {schema} {key} {value}'.format(
        schema=GSETTINGS_SCHEMA, key=key, value=str(enable).lower())
    subprocess.check_call(shlex.split(command))


def _is_auto_capitalization_enabled():
    """Return bool representing OSK auto-capitalization settings state."""
    return _is_setting_for_key_enabled(key=AUTO_CAPITALIZATION_KEY)


def _set_auto_capitalization(enable=True):
    """
    Enable/Disable auto-capitalization through gsettings.
    :param enable: boolean enable/disable auto-capitalization
    """
    _set_keyboard_setting(AUTO_CAPITALIZATION_KEY, enable=enable)


class EnableAutoCapitalization(fixtures.Fixture):
    """Enable auto-capitalization of OSK, if its not already."""

    def setUp(self):
        super().setUp()
        if not _is_auto_capitalization_enabled():
            _set_auto_capitalization(enable=True)
            self.addCleanup(_set_auto_capitalization, enable=False)


def _is_auto_correction_enabled():
    """Return bool representing OSK auto-correction setting state."""
    return _is_setting_for_key_enabled(key=AUTO_CORRECTION_KEY)


def _set_auto_correction(enable=True):
    """
    Enable/Disable auto-correction through gsettings.
    :param enable: boolean enable/disable auto-correction
    """
    _set_keyboard_setting(AUTO_CORRECTION_KEY, enable=enable)


class EnableAutoCorrection(fixtures.Fixture):
    """Enable auto-correction of OSK if its not already."""

    def setUp(self):
        super().setUp()
        if not _is_auto_correction_enabled():
            _set_auto_correction(enable=True)
            self.addCleanup(_set_auto_correction, enable=False)


def _is_words_suggestion_enabled():
    """Return bool representing OSK auto-suggestion setting state."""
    return _is_setting_for_key_enabled(key=WORDS_SUGGESTIONS_KEY)


def _set_words_suggestions(enable=True):
    """
    Switch word suggestions through gsettings.
    :param enable: boolean enable/disable auto-capitalization
    """
    _set_keyboard_setting(WORDS_SUGGESTIONS_KEY, enable=enable)


class EnableWordsSuggestions(fixtures.Fixture):
    """Enable words suggestion of OSK, if its not already."""

    def setUp(self):
        super().setUp()
        if not _is_words_suggestion_enabled():
            _set_words_suggestions(enable=True)
            self.addCleanup(_set_words_suggestions, enable=False)
