# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from retrying import retry

from autopilot.input._common import get_center_point
from unity8 import dash as unity8_dash

from ubuntu_system_tests.helpers.autopilot import (
    patched_wait_select_single, validate_dbus_object, wait_select_many)
from ubuntu_system_tests.helpers.autopilot.list import order_by_x_coord
from ubuntu_system_tests.helpers.ubuntuuitoolkit.pageheader import (
    DashPageHeader
)
from ubuntu_system_tests.helpers.unity8 import UNITY8_DASH_PATH_ROOT

SCOPE_CLASS_NAME = 'GenericScopeView'


class Dash(unity8_dash.Dash):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_DASH_PATH_ROOT, b'Dash', objectName='dash')

    def wait_for_processing_to_complete(self, timeout=90):
        """Wait until the dash is fully loaded"""
        self.select_single(
            objectName='processingIndicator').visible.wait_for(False,
                                                               timeout=timeout)

    def wait_for_dash_loaded(self):
        """Wait until the dash content is ready"""
        dash_content = self.select_single(objectName='dashContent')
        dash_content.pageHeaderTotallyVisible.wait_for(True)

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=lambda exception: (
               isinstance(exception, AssertionError)))
    def _scroll_to_left_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_left_scope()

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=lambda exception: (
               isinstance(exception, AssertionError)))
    def _scroll_to_right_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_right_scope()

    def scroll_to_left_scope(self):
        self._scroll_to_left_scope()
        self.dash_content_list.moving.wait_for(False)

    def scroll_to_right_scope(self):
        self._scroll_to_right_scope()
        self.dash_content_list.moving.wait_for(False)

    def get_current_scope(self, cpo=None, *args, **kwargs):
        """
        Return the GenericScopeView proxy object for current scope.

        :param cpo: The required scope proxy object. If this is None the
                    returned CPO will be generic.
        :return: The required scope proxy object.

        """
        if not cpo:
            from ubuntu_system_tests.helpers.scopes.generic._cpo import (
                GenericScopeView)
            cpo = GenericScopeView
        return patched_wait_select_single(
            root=self, class_object=cpo, class_name=SCOPE_CLASS_NAME,
            isCurrent=True, *args, **kwargs)

    def get_scope_count(self):
        """Count how many scopes are present."""
        return len(self.dash_content_list.select_many(SCOPE_CLASS_NAME))

    def get_current_page_header(self, header_class=DashPageHeader,
                                wait_for_dash_loaded=True):
        """
        Return the PageHeader object representing the current header
        being displayed.

        :return: PageHeader object

        """
        if wait_for_dash_loaded:
            self.wait_for_dash_loaded()

        # Get a list of headers displayed on the screen
        headers = wait_select_many(self, type_name=header_class, visible=True)

        # Return the first visible header ordered by x co-ordinate
        return order_by_x_coord(headers)[0]

    def swipe_screen_from_left(self):
        """Move a focused application to the background and return to dash."""
        width = self.width
        height = self.height
        start_x = 0
        start_y = int(height/2)
        end_x = width
        end_y = start_y
        self.pointing_device.drag(start_x, start_y, end_x, end_y)

    def open_manage_scopes(self):
        """Open and return a helper for the scope management page."""
        from ubuntu_system_tests.helpers.scopes._scopes_list import ScopesList
        overview_drag_handle = self.wait_select_single(
            objectName='overviewDragHandle')
        start_x, start_y = get_center_point(overview_drag_handle)
        stop_x = start_x
        stop_y = 0
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        self.wait_for_dash_loaded()
        return self.wait_select_single(
            ScopesList,
            objectName='scopesList',
            visible=True)
