# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import random
import time

import fixtures
from ubuntuuitoolkit import fixture_setup as ubuntuuitoolkit_fixtures

from ubuntu_system_tests.helpers import sqlite
from ubuntu_system_tests.helpers.backup_restore_fixture import (
    BackupRestoreFixture)
from ubuntu_system_tests.helpers.download_manager import (
    BackupRestoreDownloadManager)
from ubuntu_system_tests.helpers.file_system import (
    DIR_HOME_LOCAL_SHARE_WEBBROWSER)
from ubuntu_system_tests.helpers.gallery.fixture_setup import (
    BackupRestoreGalleryData)
from ubuntu_system_tests.helpers.media import BackupRestoreLocalMediaFixture
from ubuntu_system_tests.helpers.webbrowser import APP


class WebbrowserTestEnvironment(fixtures.Fixture):
    """Fixture to prepare webbrowser test environment."""

    def setUp(self):
        super().setUp()
        self.useFixture(BackupRestoreWebbrowserData())
        self.useFixture(BackupRestoreLocalMediaFixture())
        self.useFixture(BackupRestoreDownloadManager())
        self.useFixture(BackupRestoreGalleryData())


class BackupRestoreWebbrowserData(fixtures.Fixture):
    """Generic fixture class to backup and restore webbrowser data."""

    def __init__(self, start_clean=True):
        """
        :param start_clean: Whether to clean directories after backup.

        """
        self.start_clean = start_clean

    def setUp(self):
        """Backup the required directory and register restore actions."""
        super().setUp()
        self._backup_files()

    def _backup_files(self):
        """Backup the files which are removed during testing."""
        self.useFixture(BackupRestoreFixture(
            DIR_HOME_LOCAL_SHARE_WEBBROWSER, start_clean=self.start_clean))


class XdgDataHomeFixture(fixtures.Fixture):

    """Temp dir and XDG_DATA_HOME environment variable.

    Some apps (e.g. webbrowser-app) honor the freedesktop standard for
    alternate data directories [1], this provides an easy means to set
    aside data.

    1. http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html

    """
    def setUp(self):
        super().setUp()
        tempdir = self.useFixture(fixtures.TempDir())
        self.path = tempdir.path
        self.useFixture(ubuntuuitoolkit_fixtures.InitctlEnvironmentVariable(
            XDG_DATA_HOME=self.path))


class UrlDatabaseFixture(fixtures.Fixture):

    def __init__(self, urls=None, data_dir=None):
        super().__init__()
        self.data_dir = data_dir
        self.urls = urls

    def setUp(self):
        super().setUp()
        self._set_data_dir()
        self._set_db_path()
        self._ensure_db_path_exists()
        self._create_schema()
        if self.urls:
            self._populate_urls(self.urls)

    def _set_data_dir(self):
        if not self.data_dir:
            xdg_data_home = self.useFixture(XdgDataHomeFixture())
            self.data_dir = xdg_data_home.path

    def _ensure_db_path_exists(self):
        if not os.path.exists(os.path.dirname(self.db_path)):
            os.makedirs(os.path.dirname(self.db_path))

    def _populate_urls(self, urls):
        for url in urls:
            self.insert_url(**url)


class BookmarksDatabaseFixture(UrlDatabaseFixture):

    """SQLite3 database for bookmarks."""

    def _set_db_path(self):
        self.db_path = os.path.join(
            self.data_dir, APP, 'bookmarks.sqlite')

    def _create_schema(self):
        sqlite.execute(
            self.db_path,
            'CREATE TABLE IF NOT EXISTS bookmarks '
            '(url VARCHAR, title VARCHAR, icon VARCHAR, '
            'created INTEGER, folderId INTEGER);')

    def insert_url(self, **kwargs):
        """Insert a history item into the db.

        :params **kwargs: url*, title*, icon, created, folderId; *required

        """
        if not kwargs.get('created'):
            # pretend the bookmark was created a random number of seconds ago
            kwargs['created'] = int(time.time()) - int(random.random()*1000)
        for key in 'icon', 'folderId':
            if not kwargs.get(key):
                kwargs[key] = ''
        sqlite.execute(
            self.db_path,
            'INSERT INTO bookmarks VALUES '
            '(:url, :title, :icon, :created, :folderId);',
            **kwargs)


class HistoryDatabaseFixture(UrlDatabaseFixture):

    """SQLite3 database for history."""

    def _set_db_path(self):
        self.db_path = os.path.join(
            self.data_dir, APP, 'history.sqlite')

    def _create_schema(self):
        sqlite.execute(
            self.db_path,
            'CREATE TABLE IF NOT EXISTS history '
            '(url VARCHAR, domain VARCHAR, title VARCHAR, icon VARCHAR, '
            'visits INTEGER, lastVisit INTEGER);')

    def insert_url(self, **kwargs):
        """Insert a history item into the db.

        :params **kwargs: url*, domain*, title*, icon, visits, lastVisit;
            *required

        """
        if not kwargs.get('visits'):
            kwargs['visits'] = int(random.random()*4)
        if not kwargs.get('lastVisit'):
            # pretend the page was visited a random number of seconds ago
            kwargs['lastVisit'] = int(time.time()) - int(random.random()*1000)
        if not kwargs.get('icon'):
            kwargs['icon'] = ''
        sqlite.execute(
            self.db_path,
            'INSERT INTO history VALUES '
            '(:url, :domain, :title, :icon, :visits, :lastVisit);',
            **kwargs)
