# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import psutil
from retrying import retry
import stat
from tempfile import mkstemp
import threading

from autopilot import (
    exceptions,
    get_test_configuration,
    input,
    logging as autopilot_logging,
    testcase
)
from autopilot.matchers import Eventually
from autopilot._fixtures import OSKAlwaysEnabled
from autopilot.introspection import _object_registry as object_registry
from testtools.matchers import Equals
from ubuntuuitoolkit import fixture_setup
from unity8 import process_helpers
from unity8.sensors import FakePlatformSensors

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import images
from ubuntu_system_tests.helpers import mir
from ubuntu_system_tests.helpers import screen
from ubuntu_system_tests.helpers import unity8
from ubuntu_system_tests.helpers.unity8 import get_dash
from ubuntu_system_tests.helpers.unity8.shell import Unity8  # NOQA
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers import webapp

# Any test name ending with this will leave device locked
TEST_ID_DEVICE_LOCKED = 'device_locked'

logger = logging.getLogger(__name__)


class BaseUbuntuSystemTestCase(testcase.AutopilotTestCase):

    @classmethod
    def setUpClass(cls):
        unity8.ensure_unity_stopped()
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        unity8.ensure_unity_running_and_greeter_hidden()

    def setUp(self):
        super().setUp()
        self._reset_autopilot_registry()
        self._init_shared_context()

    def _init_shared_context(self):
        # The shared context is reset for each test
        context.shared.clear()
        context.shared.add_cleanup = self.addCleanup
        context.shared.timers = []

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_delay=30000,
           retry_on_exception=lambda exception: (
                isinstance(exception, psutil.NoSuchProcess) or
                isinstance(exception, ValueError)))
    def _get_unity8_proxy_object(self):
        return autopilot.get_job_proxy_object('unity8')

    @autopilot_logging.log_action(logger.info)
    def launch_unity(self, fake_sensors=False):
        """Launch Unity8 with testability enabled.

        :return: The Unity8 autopiliot custom proxy object.

        """
        self.useFixture(fixture_setup.InitctlEnvironmentVariable(
            QT_LOAD_TESTABILITY='1'))
        if fake_sensors:
            self.useFixture(fixture_setup.InitctlEnvironmentVariable(
                UBUNTU_PLATFORM_API_TEST_OVERRIDE='sensors')
            )
            # This has to be done in a seperate thread to avoid blocking
            # when creating the sensors
            launch_thread = threading.Thread(
                target=self._start_unity_with_testability
            )
            launch_thread.start()
            self._create_sensors()
            launch_thread.join()
        else:
            self._start_unity_with_testability()

        unity = self._get_unity8_proxy_object()
        context.shared.unity = unity
        unity.wait_for_greeter_content()
        # Workaround to remove the UInput keyboard created by autopilot.
        # This stops the OSK from being displayed, so must remove it.
        self._remove_autopilot_keyboard()
        self._reset_keyboard()
        self._enable_osk_for_uitk()
        # Workaround for bug lp:1474444
        self.useFixture(OSKAlwaysEnabled())
        self.set_touch_visualization()
        return unity

    def _start_unity_with_testability(self):
        process_helpers.start_job('unity8', 'QT_LOAD_TESTABILITY=1')

    def _enable_osk_for_uitk(self):
        """Enable use of the OSK with uitk helpers."""
        os.environ['UITK_USE_MALIIT'] = '1'

    def _remove_autopilot_keyboard(self):
        """Remove the UInput simulated keyboard automatically created by
        autopilot which stops the OSK from appearing."""
        if self._kb:
            if not self._kb._device._device.device:
                self.assertTrue(
                    wait_until(self._find_keyboard),
                    'Could not find the UInput keyboard created by autopilot')
            try:
                self._kb._device._device.close()
            except OSError:
                pass
            self._kb = None

    def _find_keyboard(self):
        """Find the autopilot keyboard if it is available and save instace
        in the self._kb member variable.
        :return: True if keyboard found, False otherwise."""
        kb = self._kb._device._device._find_device()
        if kb:
            self._kb._device._device.device = kb
        return kb

    def _reset_autopilot_registry(self):
        """Reset autopilot's registry between tests to avoid CPO errors."""
        object_registry.patch_registry({})

    # Retry added until fix for bug 1511895 in ubuntu-keyboard is landed
    @retry(stop_max_attempt_number=3,
           retry_on_exception=lambda exception: (
               isinstance(exception, exceptions.ProcessSearchError) or
               isinstance(exception, exceptions.BackendException)))
    def _reset_keyboard(self):
        """Reset the Autopilot connection to the OSK"""
        input.Keyboard.create('OSK')._keyboard.reset()

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_attempt_number=3,
           retry_on_exception=lambda exception: (
               isinstance(exception, FileNotFoundError) or
               isinstance(exception, AssertionError)))
    def _create_sensors(self):
        """Create fake sensors for testing"""
        Eventually(Equals(True)).match(
            lambda: process_helpers.is_job_running('unity8')
        )
        unity_pid = process_helpers._get_unity_pid()
        fifo_path = '/tmp/sensor-fifo-{}'.format(unity_pid)
        Eventually(Equals(True)).match(
            lambda: os.path.exists(fifo_path))
        assert(stat.S_ISFIFO(os.stat(fifo_path).st_mode))
        with open(fifo_path, 'w') as fifo:
            fifo.write('create accel 0 1000 0.1\n')
            fifo.write('create light 0 10 1\n')
            fifo.write('create proximity\n')
        self.sensors = FakePlatformSensors(unity_pid)

    def restart_unity(self, fake_sensors=False):
        """
        Restart and unlock unity8

        :param fake_sensors: Whether to use the fake sensors
        :returns: The proxy object for the Unity8 shell

        """
        unity8.ensure_unity_stopped()
        unity_proxy = self.launch_unity(fake_sensors=fake_sensors)
        if self.device_unlock_required():
            unity_proxy.unlock()
            get_dash().wait_for_dash_loaded()
        return unity_proxy

    def set_touch_visualization(self):
        """Set touch visualization as required and disable on cleanup."""
        config = get_test_configuration()
        enable = config.get('touch_visualization', default=None) == 'true'
        screen.set_visualization(enable)
        if enable:
            self.addCleanup(screen.disable_touch_visualization)

    def device_unlock_required(self):
        """Return True if test requires device unlocking, False otherwise."""
        test_id = self.id().split('.')[-1]
        return not test_id.endswith(TEST_ID_DEVICE_LOCKED)

    def _get_screenshot_size(self, x, y, width, height):
        """ Calculate the screenshot size """
        unity = context.shared.unity
        if width and height:
            return width, height
        else:
            return unity.get_width(), unity.get_height()

    def get_screenshot_image(self, x=0, y=0, width=0, height=0):
        """ Take an screenshot and return a PIL Image file. The screenshot
        will be automatically deleted after the test execution. Passing the
        parameters for the rectangle coordinates, the screenshot captured will
        be just for the section selected
        :return: A PIL Image file which contains the screenshot
        """
        file_path = mkstemp()[1]
        self.addCleanup(os.remove, file_path)
        mir.get_rgba_screenshot(file_path, x, y, width, height)

        return images.get_image_from_rgba(
            file_path, self._get_screenshot_size(x, y, width, height))

    def setup_webapp_environment(self):
        """ Setup the environment for the webapps. This method has to be
        called before launch the first web app in the test.
        """
        self.useFixture(fixture_setup.InitctlEnvironmentVariable(
            global_=True,
            UBUNTU_WEBVIEW_DEVTOOLS_HOST=webapp.DEFAULT_WEBVIEW_INSPECTOR_IP,
            UBUNTU_WEBVIEW_DEVTOOLS_PORT=str(
                webapp.DEFAULT_WEBVIEW_INSPECTOR_PORT)))
