# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil

from testtools import skipUnless

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers import timedate
from ubuntu_system_tests.helpers.scopes import go_to_photos_scope
from ubuntu_system_tests.helpers.scopes.fixture_setup import ScopesHints

from ubuntu_system_tests.tests import data as test_data
from ubuntu_system_tests.tests.scopes import BaseScopeTestCase, SKIP_REASON


class PhotosScopeTest(BaseScopeTestCase):
    """Tests for the Photos scope."""

    def setUp(self):
        self.useFixture(ScopesHints())
        super().setUp()

    def _check_pic_in_carousel(self, scope, pic_path):
        """ Check that a picture is in the carousel """
        pic = scope.get_picture_from_carousel(pic_path)
        self.assertIsNotNone(pic)
        return pic

    def _go_to_photo_preview(self, scope, pic_path):
        """ Go to the preview screen """
        scope.select_picture_from_carousel(pic_path)
        return scope.get_current_photo_preview()

    def _check_photo_preview(self, scope, pic, img_path):
        """ Check the open button, the date and time and that the picture
        which is being displayed is the correct one
        :param scope: The scope object
        :param pic: The picture object
        :param img_path: The path to the image file
        """
        # Go to the preview screen
        preview = self._go_to_photo_preview(scope, pic)

        # Verify the open button is present
        self.assertIsNotNone(preview.get_open_button())

        # Get last acess time for the picture
        last_access_time = timedate.get_utc_last_modification_time(img_path)

        # Verify the date is shown correctly
        self.assertEqual(last_access_time.date(), preview.get_date())

        # Verify the time is shown correctly
        self.assertEqual(last_access_time.time().replace(microsecond=0),
                         preview.get_time())

        # Verify the thumbnail is shown correctly
        self.assertEqual(pic.source, preview.get_thumbnail().source)

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES),
                'The SD card is not inserted in the device')
    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_displayed_from_sd_card(self):
        """ Verify that the pictures stored in the sd card are displayed in
        the photos scope and in the photo preview screen
        """
        sd_card_image = os.path.join(fs.get_media_folder_dir(fs.DIR_PICTURES),
                                     self.test_metadata.images.img2)

        shutil.copy(test_data.SRC_IMAGE_FILE_1, test_data.LOCAL_IMAGE_FILE)
        shutil.copy(test_data.SRC_IMAGE_FILE_2, sd_card_image)

        # Go to photos scope
        scope = go_to_photos_scope()
        sd_card_pic_file_path = fs.get_file_path(sd_card_image)
        internal_pic_file_path = fs.get_file_path(test_data.LOCAL_IMAGE_FILE)

        # Check the two pictures are in the carousel
        self._check_pic_in_carousel(scope, internal_pic_file_path)
        pic = self._check_pic_in_carousel(scope, sd_card_pic_file_path)
        self._check_photo_preview(scope, pic, sd_card_image)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_displayed_from_local_storage(self):
        """ Verify that the pictures stored on the device are displayed in the
        photos scope and in the photo preview screen, and that the
        gallery opens with a preview of the image

        """
        shutil.copy(test_data.SRC_IMAGE_FILE_1, test_data.LOCAL_IMAGE_FILE)

        # Go to photos scope
        scope = go_to_photos_scope()
        local_storage_pic_file_path = fs.get_file_path(
            test_data.LOCAL_IMAGE_FILE)

        # Check the local pic is in the carousel
        self._check_pic_in_carousel(scope, local_storage_pic_file_path)
        pic = self._check_pic_in_carousel(scope, local_storage_pic_file_path)

        self._check_photo_preview(scope, pic, test_data.LOCAL_IMAGE_FILE)
        preview = self._go_to_photo_preview(scope, pic)
        preview.click_open_button()

        # The gallery opens
        photo_viewer = gallery.get_gallery_photo_viewer()
        self.assertIn(local_storage_pic_file_path,
                      photo_viewer.get_image_source())

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_search_local_photo_content(self):
        """ Verify that it is possible to search a picture stored in local
        storage from the photos scope
        """
        scope_data = scopes.get_scope_data(scopes.SCOPE_ID_PHOTOS)
        image_path = fs.get_file_path(test_data.LOCAL_IMAGE_FILE)
        shutil.copy(test_data.SRC_IMAGE_FILE_1, test_data.LOCAL_IMAGE_FILE)

        scope = go_to_photos_scope()

        # Get the creation date of the picture from its preview page
        pic1 = self._check_pic_in_carousel(scope, image_path)
        preview = self._go_to_photo_preview(scope, pic1)
        date = preview.get_date()

        # Search by date, the photo has to be part of the result
        self._go_back_and_validate_header_title(scope_data.title)
        scope.search(date.strftime(scope.SEARCH_FORMAT))
        self.assertTrue(scope.is_in_search_result(image_path))

        # Search by different date, the photo has not to be part of the result
        self._go_back_and_validate_header_title(scope_data.title)
        date = date.replace(year=date.year-1)
        scope.search(date.strftime(scope.SEARCH_FORMAT))
        self.assertFalse(scope.is_in_search_result(image_path))
