# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil

from testtools import skipUnless

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers import web
from ubuntu_system_tests.helpers import webbrowser

from ubuntu_system_tests.tests import data as test_data
from ubuntu_system_tests.tests.scopes import BaseScopeTestCase, SKIP_REASON

THUMBNAILER_PREFIX = 'image://thumbnailer/file://'


class VideoScopeTest(BaseScopeTestCase):
    """Tests for the Video scope."""

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_search_local_video_content(self):
        """ Verify that it is possible to search videos stored in local
        storage from the videos scope
        """
        search_query = 'video2'
        video1_file = 'video1_test.ogv'
        video1_name = 'video1 test'
        video2_file = 'video2_test.ogv'
        video2_name = 'video2 test'

        video1 = os.path.join(fs.DIR_HOME_VIDEOS, video1_file)
        video2 = os.path.join(fs.DIR_HOME_VIDEOS, video2_file)

        shutil.copy2(test_data.SRC_VIDEO_FILE_1, video1)
        shutil.copy2(test_data.SRC_VIDEO_FILE_1, video2)

        scope = scopes.go_to_video_scope()

        scope.search(search_query)
        self.assertTrue(scope.is_in_search_result(video2_name))
        self.assertFalse(scope.is_in_search_result(video1_name))

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_display_local_video_content(self):
        """
        Verify local video content is displayed in the videos scope
        """
        video1_file = 'video1_test.ogv'
        video1_name = 'video1 test'
        video2_file = 'video2_test.ogv'
        video2_name = 'video2 test'

        video1 = os.path.join(fs.DIR_HOME_VIDEOS, video1_file)
        video2 = os.path.join(fs.DIR_HOME_VIDEOS, video2_file)

        shutil.copy2(test_data.SRC_VIDEO_FILE_1, video1)
        shutil.copy2(test_data.SRC_VIDEO_FILE_1, video2)

        scope = scopes.go_to_video_scope()

        self.dash.wait_for_dash_loaded()
        video_content = scope.get_my_videos_thumbnails()
        video_names = [video1_name, video2_name]
        video_files = [THUMBNAILER_PREFIX + video1,
                       THUMBNAILER_PREFIX + video2]
        self.assertCountEqual(video_names, video_content[0])
        self.assertCountEqual(video_files, video_content[1])

    def validate_video_information(self, summary, statistics, title, subtitle):
        """ Validates the video information """

        # Assert the summary, title and subtitle are not empty strings
        self.assertLess(1, len(summary))
        self.assertLess(1, len(title))
        self.assertLess(1, len(subtitle))

        # Assert the statistics are well formed
        stats = statistics.split()
        self.assertEqual(4, len(stats))
        self.assertEqual('☝', stats[0])
        self.assertEqual('☟', stats[2])
        self.assertLessEqual(0, int(stats[1]))
        self.assertLessEqual(0, int(stats[3]))

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_play_online_video_from_the_video_scope(self):
        """
        Verify the video scope show different online videos in the youtube
        section and it is possible reproduce one in the webbrowser
        """

        scope = scopes.go_to_video_scope()

        # Validate there are videos displayed in the youtube section
        videos = scope.get_youtube_videos()
        self.assertLessEqual(0, len(videos))

        # Go to the video preview for the first youtube video in the scope
        preview = scope.select_video(videos[0])

        # Validate the information displayed about the video is correct
        summary = preview.get_summary()
        statistics = preview.get_statistics()
        title = preview.get_title()
        subtitle = preview.get_subtitle()
        self.validate_video_information(summary, statistics, title, subtitle)

        # Validate the video online thumbnail exist
        thumbnail = preview.get_thumbnail()
        self.assertTrue(web.exists(thumbnail))

        # Validate the webbrowser is launched by clicking in the play button
        preview.play_video()
        self.assertTrue(wait_until(webbrowser.is_webbrowser_app_running))

        # Validate the url opened in the webbrowser is youtube
        browser = webbrowser.get_webbrowser_app_proxy()
        self.assertIn('youtube', browser.get_current_url())

        # Validate the webbrowser was invoked with the correct video id
        external_youtube_id = preview.get_external_id()
        pid = webbrowser.get_webbrowser_process_id()
        self.assertIn(external_youtube_id, processes.get_process_args(pid))

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_play_local_video_from_the_video_scope(self):
        """
        Verify the video scope show local videos and it is possible reproduce
        one in the media player
        """
        shutil.copy2(test_data.SRC_VIDEO_FILE_2, test_data.LOCAL_VIDEO_FILE_2)

        scope = scopes.go_to_video_scope()

        # Get the video from my videos section
        my_video = scope.get_my_videos()[0]

        # Go to the video preview and validate the thumbnail and title
        preview = scope.select_video(my_video)
        self.assertEqual(THUMBNAILER_PREFIX + test_data.LOCAL_VIDEO_FILE_2,
                         preview.get_thumbnail())
        self.assertEqual(self.test_metadata.video.video_avi.split('.')[0],
                         preview.get_title())

        # Play the video and validate it is reproduced in the media player
        preview.play_video()

        self.assertTrue(wait_until(mediaplayer.is_mediaplayer_running))
        media_player = mediaplayer.get_media_player_proxy().player
        self.assertEqual(fs.get_file_path(test_data.LOCAL_VIDEO_FILE_2),
                         media_player.source)

        # Validate the video is played and it finishes
        self.assertTrue(wait_until(media_player.is_playing))
        self.assertTrue(wait_until(media_player.is_play_finished))
