# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ubuntu_system_tests.helpers import addressbook
from ubuntu_system_tests.helpers import camera
from ubuntu_system_tests.helpers.camera import fixture_setup as camera_fixture
from ubuntu_system_tests.helpers import dialer_app
from ubuntu_system_tests.helpers import messaging
from ubuntu_system_tests.helpers.processes import is_process_running
from ubuntu_system_tests.helpers import wait_until

from ubuntu_system_tests.tests import base

APP_ID_CAMERA = 'com.ubuntu.camera_camera'
APP_ID_ADDRESS_BOOK = 'address-book-app'
APP_ID_MESSAGING = 'messaging-app'
APP_ID_DIALER = 'dialer-app'
APP_ID_DASH = 'unity8-dash'


class AppSwitcherTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(camera_fixture.SetCameraAccessRequests())
        self.shell = self.restart_unity().main_window

    def _switch_to_app(self, stage, app_name):
        """Select an app from the task switcher and check it is displayed."""
        stage.switch_to_app(app_name)
        self.assertTrue(wait_until(
            lambda: self.shell.get_current_focused_app() == app_name))

    def _close_app(self, stage, app_name):
        """Close an app using task switcher and check process is stopped."""
        app_window_name = self._get_window_name(app_name)
        self.assertTrue(is_process_running(app_name))
        app_windows = stage.get_app_window_names()
        count = len(app_windows)
        self.assertTrue(app_window_name in app_windows)
        stage.close_app(app_name)
        app_windows = stage.get_app_window_names()
        self.assertFalse(app_window_name in app_windows)
        self.assertFalse(is_process_running(app_name))
        self.assertEqual(len(app_windows), count - 1)

    def _get_window_name(self, app_name):
        """Return app window name for specified app name."""
        return 'appWindow_' + app_name

    def _get_window_names(self, app_names):
        """Return a list of window names for specified list of app names."""
        return [self._get_window_name(app_name) for app_name in app_names]

    def _launch_applications_and_switcher(self):
        """Launch test applications and open the task switcher
        :return: PhoneStage task switcher object.
        """
        dialer_app.launch_dialer_app_from_launcher()
        messaging.launch_messaging_app_from_launcher()
        addressbook.launch_address_book_from_launcher()
        camera.launch_camera_app_from_launcher()
        stage = self.shell.swipe_to_show_app_switcher()
        stage.swipe_to_top()
        apps = stage.get_app_window_names()
        expected_apps = [APP_ID_CAMERA, APP_ID_ADDRESS_BOOK, APP_ID_MESSAGING,
                         APP_ID_DIALER, APP_ID_DASH]
        expected_names = self._get_window_names(expected_apps)
        self.assertEqual(expected_names, apps)
        return self.shell.swipe_to_show_app_switcher()

    def test_app_selection(self):
        """
        Launch applications and check that it is possible to switch focus
        back to them using the task switcher.
        """
        stage = self._launch_applications_and_switcher()
        self._switch_to_app(stage, APP_ID_DIALER)
        stage = self.shell.swipe_to_show_app_switcher()
        self._switch_to_app(stage, APP_ID_CAMERA)

    def test_close_app(self):
        """
        Launch applications and check it is possible to close them using
        the swipe actions.
        """
        stage = self._launch_applications_and_switcher()
        self._close_app(stage, APP_ID_DIALER)
