# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import tempfile
from urllib import request

from testtools import skipUnless

from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import images
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers import music
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers.scopes.fixture_setup import ScopesHints
from ubuntu_system_tests.helpers import sdcard
from ubuntu_system_tests.helpers import url_dispatcher
from ubuntu_system_tests.helpers.unity8 import get_dash
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers import web

from ubuntu_system_tests.tests import base

data = load_test_metadata()

MESSAGES_INDICATOR = 'indicator-messages'
MESSAGES_INDICATOR_PAGE = 'indicator-messages-page'

SKIP_REASON = 'The SD card is not inserted in the device'
INSERT_MSG = 'detected'
REMOVE_MSG = 'removed'

SRC_AUDIO_FILE_1 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.count20.file)
SRC_AUDIO_FILE_2 = os.path.join(fs.DIR_TEST_DATA_AUDIO,
                                data.audio.mp3_file.file)
SRC_VIDEO_FILE_1 = os.path.join(fs.DIR_TEST_DATA_VIDEO,
                                data.video.video_avi)
SRC_PICTURE_FILE_1 = os.path.join(fs.DIR_TEST_DATA_IMAGES,
                                  data.images.img1)


class SDCardSystemTest(base.BaseUbuntuSystemTestCase):
    """This Class tests the sd card"""

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES), SKIP_REASON)
    def setUp(self):
        super().setUp()
        self.useFixture(ScopesHints())
        self.unity_proxy = self.restart_unity()
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        current_scope_url = get_dash().get_current_scope().get_url()
        self.addCleanup(url_dispatcher.go_to_url, current_scope_url)
        self.msg_number = 0
        self.unity_proxy.main_window.clear_notifications()

    def _remove_sd_card(self):
        """
        Remove the sd card, validate the indicator is displayed and wait until
        the indicator disappears
        :return: The removed device path
        """
        device = sdcard.remove()
        self.addCleanup(sdcard.insert, device)
        self.msg_number += 1
        indicator = indicators.open_message_indicator()
        self.assertIn(REMOVE_MSG, indicator.get_last_message(self.msg_number))
        indicator.wait_while_indicator_visible()
        return device

    def _insert_sd_card(self, device):
        """
        Insert the sd card and validate the indicator is displayed
        :param number: The number of message indicators displayed in the
        current test counting this one
        """
        sdcard.insert(device)
        self.msg_number += 1
        indicator = indicators.open_message_indicator()
        self.assertIn(INSERT_MSG, indicator.get_last_message(self.msg_number))

    def _copy_pictures(self, src_file, number):
        """ Copy several pictures to the sdcard """
        extension = os.path.splitext(src_file)[1]
        for i in range(number):
            sdcard.copy_picture(src_file,
                                dst_name='pic_{}{}'.format(i, extension))

    def _copy_new_pictures(self, number):
        """ Create several pictures to the sd card that will appear in the
        today category
        """
        extension = os.path.splitext(SRC_PICTURE_FILE_1)[1]
        new_file = tempfile.mkstemp(suffix=extension)[1]

        images.create_new_image(SRC_PICTURE_FILE_1, new_file)
        self.addCleanup(os.remove, new_file)
        self._copy_pictures(new_file, number)

    def _copy_unknown_pictures(self, number):
        """ Copy several pictures to the sd card that will appear in the
        unknown category. The src picture is downloaded from the web.
        """
        src_url = 'http://people.canonical.com/~platform-qa/ubuntu.jpg'
        dst = tempfile.mkstemp(suffix='.jpg')[1]

        unknown_file = web.download_file(src_url, save_location=dst)
        self.addCleanup(os.remove, unknown_file)
        self._copy_pictures(unknown_file, number)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_insert_sd_card_and_explore(self):
        """ Verify music stored in the sd card is displayed and can be played
        in the music scope when the sd card is inserted.
        """

        # Prerequisite: Copy music and remove the sd card
        sdcard.copy_music(SRC_AUDIO_FILE_1)
        device = self._remove_sd_card()

        # Go to the music scope
        scope = scopes.go_to_music_scope()

        # Check there are not any music displayed in my music section
        self.assertFalse(scope.is_my_music_displayed())

        # Insert the sd card and validate the indicator is displayed
        self._insert_sd_card(device)

        # Check the song copied to the sd card is displayed in my music section
        self.assertEqual(1, len(scope.get_my_music_titles(count=1)))

        # Play the song and check the song is being played
        artist_preview = scope.select_artist(data.audio.count20.artist)
        track_preview = artist_preview.select_track(data.audio.count20.title)
        track_preview.play()
        self.assertTrue(track_preview.is_progress_bar_moving())
        track_preview.pause()

    def test_playback_video_from_sd_card(self):
        """ Verify a video stored in the sd card can be reproduced in the
        media player
        """

        # Prerequisite: Copy video to the sd card
        dst_video = sdcard.copy_video(SRC_VIDEO_FILE_1)

        mediaplayer.play_video(dst_video)
        wait_until(mediaplayer.is_mediaplayer_running)
        player = mediaplayer.get_media_player_proxy().player

        self.assertTrue(wait_until(player.is_playing))

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_insert_sd_card_containing_new_videos(self):
        """ Verify a video stored is displayed when the sd card is inserted
        and can be reproduced in the media player from the video scope
        """

        # Prerequisite: Copy video and remove the sd card
        sdcard.copy_video(SRC_VIDEO_FILE_1)
        device = self._remove_sd_card()

        # Go to the video scope
        scope = scopes.go_to_video_scope()

        # Check there are not any video displayed in my videos section
        self.assertFalse(scope.is_my_videos_displayed())

        # Insert the sd card and validate the indicator is displayed
        self._insert_sd_card(device)

        # Check the video copied to the sd card is displayed in my videos
        my_videos = scope.get_my_videos()
        self.assertEqual(1, len(my_videos))

        # Go to the video preview and play the video
        preview = scope.select_video(my_videos[0])
        preview.play_video()

        wait_until(mediaplayer.is_mediaplayer_running)
        player = mediaplayer.get_media_player_proxy().player

        self.assertTrue(wait_until(player.is_playing))

    def test_pictures_on_sd_card_can_be_viewed(self):
        """ Verify a picture stored in the sd card can be viewed in the events
        view and can be fully displayed as well
        """
        # Prerequisite: Copy picture to the sd card
        dst_pic = sdcard.copy_picture(SRC_PICTURE_FILE_1)

        gallery_app = gallery.launch_gallery_app()
        self.assertTrue(gallery_app.is_image_displayed(dst_pic))

        gallery_app.click_photo(dst_pic)
        self.assertTrue(gallery_app.is_image_in_view(dst_pic))

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_play_music_from_sd_card_in_music_app(self):
        """ Verify it is possible to reproduce music stored in the sd card
        from the music scope
        """
        title = data.audio.count20.title
        artist = data.audio.count20.artist
        filename = data.audio.count20.file

        # Prerequisite: Copy songs with same artist and album to the sd card
        sdcard.copy_music(SRC_AUDIO_FILE_1)
        sdcard.copy_music(SRC_AUDIO_FILE_2)

        scope = scopes.go_to_music_scope()
        # Go to the track preview
        preview = scope.select_artist(artist).select_track(title)

        # Play the album in the music app and verify the desired album is
        # being reproduced
        preview.play_in_music_app()
        self.assertTrue(wait_until(music.is_music_app_running, timeout=20))
        self.addCleanup(music.stop_music_app)
        song_path = os.path.join(
            fs.DIR_MEDIA_ROOT, sdcard.get_sd_card_dir_name(), 'Music',
            filename)
        self.assertEqual(
            request.pathname2url(song_path), music.get_current_song())

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_view_photos_from_sdcard(self):
        """ Validate pictures stored in the sd card are displayed in the photos
        scope and the buttons "See all" and "See less" work properly
        """

        # This step is added at the beginning to disable the skip button if
        # needed, this is done before copying the pictures to the sd card to
        # see those correctly in the 'my photos' section
        scopes.go_to_photos_scope()

        # Prerequisite: Copy several pictures to the sd card
        self._copy_new_pictures(20)
        self._copy_unknown_pictures(2)

        # Reload scope and go to my photos section
        scope = scopes.go_to_photos_scope()
        my_photos = scope.go_to_my_photos()

        today = my_photos.get_today_category()
        # Check all the pictures are displayed clicking on "See all"
        self.assertTrue(today.is_see_all_displayed())
        my_photos.see_all(today)
        self.assertLessEqual(20, len(today.get_pictures_list()))
        self.assertTrue(today.expanded)

        # Check the pictures section is not expanded clicking on "See less"
        self.assertTrue(today.is_see_less_displayed())
        my_photos.see_less(today)
        self.assertTrue(wait_until(today.is_not_see_less_displayed))
