# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright 2014 Canonical Ltd.
#
# This file is part of ubuntu-experience-tests.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>

from ubuntu_system_tests.config import get_device_config_stack
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers.platform import (
    is_agps_supported, is_wifi_supported
)
from ubuntu_system_tests.helpers import testbed
from ubuntu_system_tests.helpers.unity8.fixture_setup import SettingsWizard
from ubuntu_system_tests.helpers.unity8 import settings_wizard
from ubuntu_system_tests.helpers.unity8.settings_wizard import Wizard

from ubuntu_system_tests.tests import base

KEY_WIFI_SSID = 'wifi_ssid'
KEY_WIFI_PASSWORD = 'wifi_password'
KEY_SECURITY = 'device_security'
KEY_DEFAULT_SECURITY = 'default_device_security'
KEY_DEVICE_PASSWORD = 'device_password'

SECURITY_PASSCODE = 'Passcode'
SECURITY_PASSPHRASE = 'Passphrase'

DEFAULT_LANGUAGE = 'English (United States)'
DEFAULT_SECURITY = SECURITY_PASSCODE


class SettingsWizardTestCase(base.BaseUbuntuSystemTestCase):
    """Autopilot test for completing settings wizard"""

    def setUp(self):
        """Setup required configuration for starting test"""
        super().setUp()
        self._apply_default_config()
        self.wizard_helper = self.useFixture(SettingsWizard(True))
        self.unity = self.launch_unity()
        self.wizard = self._get_settings_wizard()
        self.is_sim_inserted = ofono.is_sim_inserted(ignore_phonesim=False)
        self.useFixture(testbed.RemoveNetworks())

    def _apply_default_config(self):
        """Create config for the test based on supplied and default values"""
        self.config = {}
        config = get_device_config_stack()
        self.config[KEY_WIFI_SSID] = config.get(KEY_WIFI_SSID)
        self.config[KEY_WIFI_PASSWORD] = config.get(KEY_WIFI_PASSWORD)
        self.config[KEY_DEVICE_PASSWORD] = config.get(KEY_DEVICE_PASSWORD)
        self.config[KEY_SECURITY] = config.get(KEY_SECURITY, DEFAULT_SECURITY)

    def _get_settings_wizard(self):
        """Wait for the settings wizard to appear and return Wizard object"""
        return self.unity.wait_select_single(Wizard)

    def _test_language_page(self):
        """Get the language page, check default language and continue"""
        language_page = self.wizard.get_languge_page()
        # check the default language is selected
        default_selection = language_page.get_selected_language()
        self.assertEqual(default_selection, DEFAULT_LANGUAGE)
        # press continue and move to next page
        # this might be the 'no sim' page, or security settings
        return language_page.continue_()

    def _test_sim_inserted(self, sim_inserted, page):
        """Check if sim page is diaplayed and test it."""
        self.assertEqual(sim_inserted, self.is_sim_inserted)
        if sim_inserted:
            # page is already the password page
            password_page = page
        else:
            # the page tells us that no sim is inserted
            # press the skip button
            password_page = page.skip()
        return password_page

    def _complete_security_page(self, page, security_option):
        """Complete the security page using the requested option."""
        if security_option == SECURITY_PASSCODE:
            # enter passcode and confirm
            enter_pin_page = self._test_password_page(page)
            confirm_pin_page = enter_pin_page.enter_pin(
                self.config[KEY_DEVICE_PASSWORD])
            wifi_connect_page = confirm_pin_page.enter_pin(
                self.config[KEY_DEVICE_PASSWORD])
        elif security_option == SECURITY_PASSPHRASE:
            # enter password, continue and confirm
            enter_password_page = self._test_password_page(page)
            enter_password_page.enter_text(
                self.config[KEY_DEVICE_PASSWORD])
            confirm_password_page = enter_password_page.continue_()
            confirm_password_page.enter_text(
                self.config[KEY_DEVICE_PASSWORD])
            wifi_connect_page = confirm_password_page.continue_()
        else:
            # this is swipe, just continue
            wifi_connect_page = self._test_password_page(page)
        return wifi_connect_page

    def _test_password_page(self, password_page):
        """Check default security option and continue"""
        # check default security is assigned
        default_selection = password_page.get_selected_security_option()
        self.assertEqual(default_selection, DEFAULT_SECURITY)
        # select a different security option if not using the default one
        if self.config[KEY_SECURITY] != default_selection:
            password_page.select_security_option(self.config[KEY_SECURITY])
        # move onto security page using default security method
        return password_page.continue_()

    def _test_wifi_connect_page(self, wifi_connect_page, ssid, password):
        """Select Wi-Fi network and enter password if not setup already"""
        # If wi-fi is supported and no existing wi-fi network
        # is already setup, then set a new wi-fi network up.
        if (is_wifi_supported() and
           not wifi_connect_page.is_network_checked(ssid)):
            # select the network and enter password
            password_notification = wifi_connect_page.select_network(
                self.unity, ssid)
            password_notification.enter_text(password)
            password_notification.connect()
            # check that the network is now checked
            self.assertTrue(wifi_connect_page.is_network_checked(ssid))
        return wifi_connect_page.continue_()

    def _test_location_support(self, location_supported, page):
        """Check if location page is displayed and test it."""
        self.assertEqual(location_supported, is_agps_supported())
        if location_supported:
            # test the location page and return reporting page
            reporting_page = self._test_location_page(page)
        else:
            # page is already the finish page
            reporting_page = page
        return reporting_page

    def _test_location_page(self, location_page):
        """Check default setting and move onto next page."""
        self.assertEqual(
            location_page.get_selected_option(), location_page.OPTION_GPS_ALL)
        return location_page.continue_()

    def _test_reporting_page(self, reporting_page):
        """Continue onto next page"""
        return reporting_page.continue_()

    def _test_finish_page(self, finish_page):
        """Finish wizard and check it is closed"""
        finish_page.finish()

    def test_complete_settings_wizard(self):
        """Test completion of the settings wizard using default settings"""
        # start with the language page
        sim_inserted, next_page = self._test_language_page()

        # check if sim page is displayed, return password page
        password_page = self._test_sim_inserted(sim_inserted, next_page)

        # complete the security page
        wifi_connect_page = self._complete_security_page(
            password_page, self.config[KEY_SECURITY])

        # complete the location page
        location_supported, next_page = self._test_wifi_connect_page(
            wifi_connect_page,
            self.config[KEY_WIFI_SSID],
            self.config[KEY_WIFI_PASSWORD])

        if settings_wizard.reporting_page_included():
            # Validate the reporting page when  it is part of the wizard
            reporting_page = self._test_location_support(
                location_supported, next_page)
            finish_page = self._test_reporting_page(reporting_page)
        else:
            finish_page = self._test_location_support(
                location_supported, next_page)

        self._test_finish_page(finish_page)

        # check that the wizard is now disabled
        self.assertFalse(
            self.wizard_helper.is_settings_wizard_enabled())
