# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from autopilot.matchers import Eventually
from testtools.matchers import Equals

from ubuntu_keyboard.emulators.keyboard import Keyboard as UbuntuKeyboard

from ubuntu_system_tests.tests import base
from ubuntu_system_tests.helpers.messaging import(
    fixture_setup as messaging_fixture
)
from ubuntu_system_tests.helpers import(
    messaging as messaging_helpers,
    ubuntu_keyboard as osk_helpers
)
from ubuntu_system_tests.helpers.ubuntu_keyboard import(
    fixture_setup as keyboard_fixture
)

INITIAL_TEXT = 'Hello my frien'
SENTENCE_1 = 'Hello my friends '
EXPECTED_WORDS = ['friends', 'friend', 'friendship', 'friendly']
UNIQUE_WORD_1 = 'blahblahxx'
UNIQUE_WORD_2 = 'asdfgh'


class KeyboardBase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.unity_proxy = self.restart_unity()
        self.useFixture(messaging_fixture.SetDefaultSIM())
        self.messaging_app = messaging_helpers.launch_messaging_app()

    def _focus_message_input_field_to_invoke_keyboard(self):
        # Due to some reason, using the proxy object with uitk emulator base,
        # causes the Ubuntu Keyboard to misbehave i.e. its auto-capitalization
        # settings are reset during the test. Getting the proxy object without
        # custom emulator base, solves the issue. --om26er 16-12-2015
        messaging_app_raw_proxy = messaging_helpers.get_messaging_app_proxy(
            base=None)
        return messaging_helpers.focus_message_input_field(
            messaging_app_raw_proxy, self.messaging_app.pointing_device)


class KeyboardWithAutoCapitalizationTestCase(KeyboardBase):

    def setUp(self):
        super().setUp()
        self.useFixture(keyboard_fixture.EnableAutoCapitalization())
        self.messaging_app.start_new_message()

    def test_auto_capitalization(self):
        """
        1. Open messaging app and create a new message.
        2. Type 'Hello my friend. ' into the message field and ensure
           auto-capitalization works.
        3. Remove the trailing white space after the '.' and ensure the
           keyboard no longer has the shift key enabled.
        """
        # Focus message input field and ensure the OSK appears with
        # auto-capitalization enabled
        self._focus_message_input_field_to_invoke_keyboard()
        self.assertThat(lambda: osk_helpers.is_keyboard_shifted(),
                        Eventually(Equals(True)))

        keyboard = UbuntuKeyboard()
        keyboard.type('Hello my friend. ')
        self.assertThat(lambda: osk_helpers.is_keyboard_shifted(),
                        Eventually(Equals(True)))

        keyboard.type('\b')
        self.assertThat(lambda: osk_helpers.is_keyboard_shifted(),
                        Eventually(Equals(False)))


class KeyboardWithWordsSuggestionsTestCase(KeyboardBase):

    def setUp(self):
        super().setUp()
        self.useFixture(keyboard_fixture.EnableAutoCorrection())
        self.useFixture(keyboard_fixture.EnableWordsSuggestions())
        self.messaging_app.start_new_message()
        self.input_field = self._focus_message_input_field_to_invoke_keyboard()

    def _assert_words_in_ribbon(self, words_in_ribbon, expected_words_list):
        for word in expected_words_list:
            self.assertTrue(word in words_in_ribbon)

    def test_words_suggestions(self):
        """
        1. Type some text into an input field and check similar words appear
           there
        2. Press space and ensure words-ribbon gets cleared and previously bold
           text appears into the input field
        3. Type a unique word like 'blahblahxx' and ensure it shows bold in
           ribbon
        4. Tap on that word and ensure its added to the input field
        5. Type another unique word like 'asdfgh' and ensure it shows bold
        6. Press a space and type 'i' and ensure that the word 'I' is shown
           capitalized as bold
        """
        osk = UbuntuKeyboard()
        osk.type(INITIAL_TEXT)
        proxy = osk_helpers.get_ubuntu_keyboard_proxy()
        word_ribbon_list = proxy.keyboard.word_ribbon_list

        suggestions_list = word_ribbon_list.get_suggested_words()
        self._assert_words_in_ribbon(suggestions_list, EXPECTED_WORDS)

        osk.type(' ')
        self.assertThat(self.input_field.text, Eventually(Equals(SENTENCE_1)))
        self.assertThat(lambda: word_ribbon_list.is_suggestions_list_empty(),
                        Eventually(Equals(True)))

        osk.type(UNIQUE_WORD_1)
        self.assertThat(lambda: word_ribbon_list.is_text_bold(UNIQUE_WORD_1),
                        Eventually(Equals(True)))
        word_ribbon_list.tap_suggested_word(text=UNIQUE_WORD_1)
        expected_text = '{}{} '.format(SENTENCE_1, UNIQUE_WORD_1)
        self.assertThat(self.input_field.text,
                        Eventually(Equals(expected_text)))

        osk.type(UNIQUE_WORD_2)
        self.assertThat(lambda: word_ribbon_list.is_text_bold(UNIQUE_WORD_2),
                        Eventually(Equals(True)))

        osk.type(' i')
        self.assertThat(lambda: word_ribbon_list.is_text_bold('I'),
                        Eventually(Equals(True)))
